#
# NuGridpy - Tools for accessing and visualising NuGrid data.
#
# Copyright 2007 - 2014 by the NuGrid Team.
# All rights reserved. See LICENSE.
#

''' 
MESA output data loading and plotting

v0.2, 15OCT2012: NuGrid collaboration
(Sam Jones, Michael Bennett, Daniel Conti, William Hillary,
Falk Herwig, Christian Ritter)

v0.1, 23JUN2010: Falk Herwig

    mesa.py provides tools to get MESA stellar evolution data output
    into your favourite python session.  In the LOGS directory MESA
    outputs two types of files: history.data or star.log is a time
    evolution output, printing one line per so many cycles (e.g. each
    cycle) of all sorts of things.  profilennn.data or lognnn.data
    files are profile data files.  nnn is the number of profile.data
    or log.data files that is translated into model cycles in the
    profiles.index file.

    MESA allows users to freely define what should go into these two
    types of outputs, which means that column numbers can and do
    change.  mesa.py reads in both types of files and present them (as
    well as any header attributes) as arrays that can be referenced by
    the actual column name as defined in the header section of the
    files.  mesa.py then defines a (hopefully growing) set of standard
    plots that make use of the data just obtained.

    mesa.py is organised as a module that can be imported into any
    python or ipython session. It is related to nugridse.py which is a
    similar module to deal with 'se' output, used by the NuGrid
    collaboration.  mesa.py does not need se libraries.  The 'se' output
    files that can be written with MESA can be read and processed with
    the nugridse.py tool.

    mesa.py is providing two class objects, mesa_profile and
    history_data.  The first makes profile data available, the second
    reads and plots the history.data or star.log file.  Note that
    several instances of these can be initiated within one session and
    data from different instances (i.e. models, tracks etc) can be
    overplotted.

    Here is how a simple session could look like that is plotting an
    HRD (We prefer to load ipython with matplotlib and numpy support
    via the alias:
    
    alias mpython='ipython --pylab')

    >>> import mesa as ms
    >>> help ms
    ------> help(ms)

    >>> s=ms.history_data('.')
    >>> s.hrd()

    In order to find out what header attributes and columns are
    available in history.data or star.log use:

    >>> s.header_attr
    {'burn_min1': 50.0,
     'burn_min2': 1000.0,
     'c12_boundary_limit': 0.0001,
     'h1_boundary_limit': 0.0001,
     'he4_boundary_limit': 0.0001,
     'initial_mass': 2.0,
     'initial_z': 0.01}

    >>> s.cols
    {'center_c12': 38,
     'center_h1': 36,
     'center_he4': 37,
     ...

    In order to read the profile data from the first profile.data file
    in profiles.index, and then get the mass and temperature out and
    finally plot them try.  Typically you will have already a
    Kippenhahn diagram as a function of model number in front of you,
    and you want to access profile information for a given cycle
    number.  Typically you do not have profiles for all cycle
    numbers.  The best way to start a profile instance is with
    num_type='nearest_model' (check the docstring for other ways to
    select profiles for a profile instance):

    >>> a1=ms.mesa_profile('LOGS',59070)
    2001 in profiles.index file ...
    Found and load nearest profile for cycle 59000
    reading LOGS/profile1801.data ...
    Closing profile tool ...

    >>> T=a1.get('temperature')
    >>> mass=a1.get('mmid')
    >>> plot(mass,T)
    [<matplotlib.lines.Line2D object at 0x8456ed0>]

    Or, you could have had it easier in the following way:
    
    >>> a1.plot('mass','c12',logy=True,shape='-',legend='$^{12}\mathrm{C}$')

    where the superclass plot method interprets data column headers
    correctly and does all the work for you.

    Of course, a1.cols etc are available here as well and many other
    things. E.g. a.model contains an array with all the models for
    which profile.data or log.data  are available. You may initiate a profile object
    with a model number:

    >>> a2=ms.mesa_profile('.',55000,num_type='model')
    100 in profiles.index file ...
    reading ./profile87.data ...

    a1.log_ind (for any profile instance) provides a map of model
    number to profile file number.
    a1.cols and a1.header_attr gives the column names and header attributes.
   
'''
import ascii_table
import numpy as np
import utils as u
from data_plot import *
import numpy as np
import matplotlib
import matplotlib.pylab as pyl
import matplotlib.pyplot as pl
from matplotlib.collections import Collection
from matplotlib.artist import allow_rasterization
from matplotlib.patches import PathPatch
import os
import sys

def set_nugrid_path(path):
    '''
        This function sets the path to the NuGrid VOSpace directory as a
        global variable, so that it need only be set once during an inter-
        active session.
        '''
    global nugrid_path
    nugrid_path=path

def set_nice_params():
    fsize=18
        
    params = {'axes.labelsize':  fsize,
    #    'font.family':       'serif',
    'font.family':        'Times New Roman',
    'figure.facecolor':  'white',
    'text.fontsize':     fsize,
    'legend.fontsize':   fsize,
    'xtick.labelsize':   fsize*0.8,
    'ytick.labelsize':   fsize*0.8,
    'ytick.minor.pad': 8,
    'ytick.major.pad': 8,
    'xtick.minor.pad': 8,
    'xtick.major.pad': 8,
    'text.usetex':       False}
    pl.rcParams.update(params)

class mesa_profile(DataPlot):
    ''' 
    read profiles.index and prepare reading MESA profile files

    starts with reading profiles.index and creates hash array
    profile.data can then be accessed via prof_plot

    Parameters
    ----------
    sldir : string
        Directory path of LOGS.
    num : integer
        by default this is the i.  profile file (profile.data or
        log.data) available (e.g. num=1 is the 1. available profile
        file), however if you give
    num_type : string, optional
        If 'model' (exact) or 'nearest_model': get the profile 
        profile.data file for model (or cycle number) used by the
        stellar evolution code

        If 'profile_num': num will be interpreted as the
        profile.data or log.data number profile_num (profile_num is
        the number that appears in the file names of type
        profile23.data or log23.data)

        If 'profiles_i': the ith file in profiles.index file
            
        The default is "nearest_model".
    prof_ind_name : string, optional
        Use this optional argument if the profiles.index file has
        an alternative name, for example, do
        superpro=ms.profile('LOGS',1,prof_ind_name='super.prof')
    profile_prefix : string, optional
        Prifix in the profile name.  The default is "profile".
    data_suffix : string, optional
        Optional arguments that allow you to change the defaults
        for the profile.data profile files.
    mass : integer or float, optional
        The user may select a mass and metallicity instead of providing
        the sldir explicitly, if they are using the VOSpace data. If mass
        is provided then Z should also be provided.
        The default is None (i.e. user gives sldir explicitly)
    Z : float, optional
        See 'mass' above.
        The default is None (i.e. user gives sldir explicitly)
        
        
    Examples
    --------
    initialise a mesa_profile instance for cycle 2000 like this:
        
    >>>my_profile1=ms.mesa_profile('LOGS',2000)
        
    or like this:
        
    >>>my_profile2=ms.mesa_profile(mass=2,Z=0.01,num=2000)
    '''

    sldir = ''

    def __init__(self, sldir='./LOGS', num=1, num_type='nearest_model',
                 prof_ind_name='profiles.index',
                 profile_prefix='profile', data_suffix='.data', mass=None,
                 Z=None):
        ''' 
        read a profile.data profile file

        Parameters
        ----------
        sldir : string
            Directory path of LOGS.
        num : integer
            by default this is the i.  profile file (profile.data or
            log.data) available (e.g. num=1 is the 1. available profile
            file), however if you give
        num_type : string, optional
            If 'model' (exact) or 'nearest_model': get the profile 
            profile.data file for model (or cycle number) used by the
            stellar evolution code

            If 'profile_num': num will be interpreted as the
            profile.data or log.data number profile_num (profile_num is
            the number that appears in the file names of type
            profile23.data or log23.data)

            If 'profiles_i': the ith file in profiles.index file
            
            The default is "nearest_model".
        prof_ind_name : string, optional
            Use this optional argument if the profiles.index file has
            an alternative name, for example, do
            superpro=ms.profile('LOGS',1,prof_ind_name='super.prof')
        profile_prefix : string, optional
            Prifix in the profile name.  The default is "profile".
        data_suffix : string, optional
            Optional arguments that allow you to change the defaults
            for the profile.data profile files.
        mass : integer or float, optional
            The user may select a mass and metallicity instead of providing
            the sldir explicitly, if they are using the VOSpace data. If mass
            is provided then Z should also be provided.
            The default is None (i.e. user gives sldir explicitly)
        Z : float, optional
            See 'mass' above.
            The default is None (i.e. user gives sldir explicitly)
            
        Examples
        --------
        initialise a mesa_profile instance for cycle 2000 like this:
        
        >>>my_profile1=ms.mesa_profile('LOGS',2000)
        
        or like this:
        
        >>>my_profile2=ms.mesa_profile(mass=2,Z=0.01,num=2000)

        '''

        self.prof_ind_name = prof_ind_name
        self.sldir         = sldir

        # seeker to find the data requested on VOspace:
        if mass is not None and Z is not None:
            try:
                print 'nugrid_path = '+nugrid_path
            except:
                raise IOError("nugrid_path has not been set. This is the path to the NuGrid VOSpace, e.g. /tmp/NuGrid. Set this using mesa.set_nugrid_path('path')")
            
            # which set? [find nearest]
            setsZs=[0.02,0.01,6.e-3,1.e-3,1.e-4]
            setsnames=['set1.2','set1.1','set1.3a','set1.4a','set1.5a']
            idx=np.abs(np.array(setsZs)-Z).argmin()
            setname=setsnames[idx]
            realZ=setsZs[idx]
            
            print 'closest set is '+setname+' (Z = '+str(realZ)+')'
            
            # try first data, then data-team:
            mod_dir = nugrid_path+'/data/set1/'+setname+'/see_wind/'
            if not os.path.exists(mod_dir):
                mod_dir = nugrid_path+'/data-team/Set1_extension/'+setname+'/see_wind/'
            if not os.path.exists(mod_dir):
                print 'mod_dir = ', mod_dir
                raise IOError("The data does not seem to be here. Please check that the NuGrid VOSpace is mounted and nugrid_path has been set correctly using mesa.set_nugrid_path('path')'.")
            
            # which mass? [find nearest]
            list=[el for el in os.listdir(mod_dir) if el[0]=='M']
            if len(list) == 0:
                raise IOError("Sorry. There is no data available for this set at present: "+mod_dir)
            
            setmasses=[el[1:el.index('Z')] for el in list]
            for i in range(len(setmasses)):
                if setmasses[i][-1]=='.': setmasses[i]=setmasses[i][:-1]
                setmasses[i] = float(setmasses[i])
            idx2=np.abs(np.array(setmasses)-mass).argmin()
            modname=list[idx2]
            realmass=setmasses[idx2]
            
            print 'closest mass is '+str(realmass)
            
            mod_dir+=modname
            if 'LOGS' not in os.listdir(mod_dir):
                raise IOError("No 'LOGS' directory for this model. It may have been computed with the Geneva code. Try nugridse.py to explore the see_wind data for this model.")
            else:
                self.sldir=mod_dir+'/LOGS'
                sldir = mod_dir+'/LOGS'

        
        if num_type is 'nearest_model' or num_type is 'model':
            self._profiles_index()
        if num_type is 'nearest_model':
            amods=array(self.model)
            nearmods=[where(amods<=num)[0][-1],where(amods>=num)[0][0]]
            sometable={}
            for thing in nearmods:
                sometable[abs(self.model[thing]-num)]=thing
            nearest       = min(abs(self.model[nearmods[0]]-num),\
                                    abs(self.model[nearmods[1]]-num))
            num = self.model[sometable[nearest]]
            print 'Found and load nearest profile for cycle '+str(num)
            num_type = 'model'
        if num_type is 'model':
            try:
                log_num=self.log_ind[num]
            except KeyError:
                print 'There is no profile file for this model'
                print "You may retry with num_type='nearest_model'"
                return
        elif num_type is 'profiles_i':
            log_num=self._log_file_ind(num)
            if log_num == -1:
                print "Could not find a profile file with that number"
                return
        elif num_type is 'profile_num':
            log_num = num
        else:
            print 'unknown num_type'
            return

	filename=self.sldir+'/'+profile_prefix+str(log_num)+data_suffix
        if not os.path.exists(filename):
            profile_prefix='log'
            filename=self.sldir+'/'+profile_prefix+str(log_num)+data_suffix
            if not os.path.exists(filename):
                print 'error: no profile.data file found in '+sldir
                print 'error: no log.data file found in '+sldir


        print 'reading '+filename+' ...'
        header_attr = _read_mesafile(filename,only='header_attr')
        num_zones=int(header_attr['num_zones'])
        header_attr,cols,data = _read_mesafile(filename,data_rows=num_zones,only='all')

        self.cols        = cols
        self.header_attr = header_attr
        self.data        = data


    def __del__(self):
        print 'Closing profile tool ...'

    def _profiles_index(self):
        ''' 
        read profiles.index and make hash array

        Notes
        -----
        sets the attributes.
        
        log_ind : hash array that returns profile.data or log.data
        file number from model number.
        
        model : the models for which profile.data or log.data is
        available
        
        '''

        prof_ind_name = self.prof_ind_name

        f = open(self.sldir+'/'+prof_ind_name,'r')
        line = f.readline()
        numlines=int(line.split()[0])
        print str(numlines)+' in profiles.index file ...'

        model=[]
        log_file_num=[]
        for line in f:
            model.append(int(line.split()[0]))
            log_file_num.append(int(line.split()[2]))

        log_ind={}    # profile.data number from model
        for a,b in zip(model,log_file_num):
            log_ind[a] = b

        self.log_ind=log_ind
        self.model=model

# let's start with functions that aquire data

    def _log_file_ind(self,inum):
        ''' 
        Information about available profile.data or log.data files.
        
        Parameters
        ----------
        inum : integer
            Attempt to get number of inum's profile.data file.
            inum_max: max number of profile.data or log.data files
            available
            
        '''

        self._profiles_index()
        if inum <= 0:
            print "Smallest argument is 1"
            return

        inum_max = len(self.log_ind)
        inum -= 1

        if inum > inum_max:
            print 'There are only '+str(inum_max)+' profile file available.'
            log_data_number = -1
            return log_data_number
        else:
            log_data_number=self.log_ind[self.model[inum]]
            print 'The '+str(inum+1)+'. profile.data file is '+ \
                  str(log_data_number)
            return log_data_number

    def get(self,str_name):
        ''' 
        return a column of data with the name str_name.

        Parameters
        ----------
        str_name : string
            Is the name of the column as printed in the
            profilennn.data or lognnn.data file; get the available
            columns from self.cols (where you replace self with the
            name of your instance)

        '''

        column_array = self.data[:,self.cols[str_name]-1].astype('float')
        return column_array




class history_data(DataPlot):
    ''' 
    read history.data or star.log MESA output and plot various things,
    including HRD, Kippenhahn etc

    Parameters
    ----------
    sldir : string
        which LOGS directory.
    slname : string, optional
        If star.log is available instead, star.log file is read, this
        is an optional argument if history.data or star.log file has
        an alternative name.  The default is "history.data".
    clean_starlog : boolean, optional
        Request new cleaning of history.data or star.log, makes
        history.datasa or star.logsa which is the file that is actually
        read and plotted.  The default is False.
    mass : integer or float, optional
        The user may select a mass and metallicity instead of providing
        the sldir explicitly, if they are using the VOSpace data. If mass
        is provided then Z should also be provided.
        The default is None (i.e. user gives sldir explicitly)
    Z : float, optional
        See 'mass' above.
        The default is None (i.e. user gives sldir explicitly)

    Examples
    --------
    use like this:

    >>> another=ms.history_data('LOGS',slname='anothername')
        
    or this:
        
    >>> ms.set_nugrid_path('/tmp/NuGrid')
    >>> anotherone=ms.history_data(mass=2,Z=0.01)
    '''

    sldir  = ''
    slname = ''
    header_attr = []
    cols = []

    def __init__(self, sldir='./LOGS', slname='history.data',
                 clean_starlog=False, mass=None, Z=None):
        self.sldir  = sldir
        self.slname = slname
        self.clean_starlog  = clean_starlog
        
        # seeker to find the data requested on VOspace:
        if mass is not None and Z is not None:
            try:
                print 'nugrid_path = '+nugrid_path
            except:
                raise IOError("nugrid_path has not been set. This is the path to the NuGrid VOSpace, e.g. /tmp/NuGrid. Set this using mesa.set_nugrid_path('path')")
            
            # which set? [find nearest]
            setsZs=[0.02,0.01,6.e-3,1.e-3,1.e-4]
            setsnames=['set1.2','set1.1','set1.3a','set1.4a','set1.5a']
            idx=np.abs(np.array(setsZs)-Z).argmin()
            setname=setsnames[idx]
            realZ=setsZs[idx]
            
            print 'closest set is '+setname+' (Z = '+str(realZ)+')'
            
            # try first data, then data-team:
            mod_dir = nugrid_path+'/data/set1/'+setname+'/see_wind/'
            if not os.path.exists(mod_dir):
                mod_dir = nugrid_path+'/data-team/Set1_extension/'+setname+'/see_wind/'
            if not os.path.exists(mod_dir):
                print 'mod_dir = ', mod_dir
                raise IOError("The data does not seem to be here. Please check that the NuGrid VOSpace is mounted and nugrid_path has been set correctly using mesa.set_nugrid_path('path')'.")
            
            # which mass? [find nearest]
            list=[el for el in os.listdir(mod_dir) if el[0]=='M']
            if len(list) == 0:
                raise IOError("Sorry. There is no data available for this set at present: "+mod_dir)
            
            setmasses=[el[1:el.index('Z')] for el in list]
            for i in range(len(setmasses)):
                if setmasses[i][-1]=='.': setmasses[i]=setmasses[i][:-1]
                setmasses[i] = float(setmasses[i])
            idx2=np.abs(np.array(setmasses)-mass).argmin()
            modname=list[idx2]
            realmass=setmasses[idx2]
            
            print 'closest mass is '+str(realmass)
            
            mod_dir+=modname
            if 'LOGS' not in os.listdir(mod_dir):
                raise IOError("No 'LOGS' directory for this model. It may have been computed with the Geneva code. Try nugridse.py to explore the see_wind data for this model.")
            else:
                self.sldir=mod_dir+'/LOGS'
                sldir=mod_dir+'/LOGS'
        
        if not os.path.exists(self.sldir+'/'+self.slname):
            if not os.path.exists(self.sldir+'/'+'star.log'):
                print 'error: no history.data file found in '+sldir
                print 'error: no star.log file found in '+sldir
            else:
                self.slname='star.log'
                self._read_starlog()
        else:
            self._read_starlog()

    def __del__(self):
        print 'Closing', self.slname,' tool ...'

# let's start with functions that aquire data
    def _read_starlog(self):
        ''' read history.data or star.log file again'''

        sldir   = self.sldir
        slname  = self.slname
        slaname = slname+'sa'

        if not os.path.exists(sldir+'/'+slaname):
            print 'No '+self.slname+'sa file found, create new one from '+self.slname
            _cleanstarlog(sldir+'/'+slname)
        else:
            if self.clean_starlog:
                print 'Requested new '+self.slname+'sa; create new from '+self.slname
                _cleanstarlog(sldir+'/'+slname)
            else:
                print 'Using old '+self.slname+'sa file ...'

        cmd=os.popen('wc '+sldir+'/'+slaname)
        cmd_out=cmd.readline()
        cnum_cycles=cmd_out.split()[0]
        num_cycles=int(cnum_cycles) - 6

        filename=sldir+'/'+slaname

        header_attr,cols,data = _read_mesafile(filename,data_rows=num_cycles)

        self.cols        = cols
        self.header_attr = header_attr
        self.data        = data

    def get(self, str_name):
        ''' 
        return a column of data with the name str_name.

        Parameters
        ----------
        str_name : string
            The name of the column as printed in history.data or
            star.log get the available columns from self.cols (where
            you replace self with the name of your instance
            
        '''

        column_array = self.data[:,self.cols[str_name]-1].astype('float')
        return column_array

    def CO_ratio(self,ifig,ixaxis):
        '''
        plot surface C/O ratio in Figure ifig with x-axis quantity ixaxis
            
        Parameters
        ----------
        ifig : integer
            Figure number in which to plot
        ixaxis : string
            what quantity is to be on the x-axis, either 'time' or 'model'
            The default is 'model'
        '''
    
        def C_O(model):
            surface_c12=model.get('surface_c12')
            surface_o16=model.get('surface_o16')
            CORatio=(surface_c12*4.)/(surface_o16*3.)
            return CORatio

        if ixaxis=='time':
            xax=self.get('star_age')
        elif ixaxis=='model':
            xax=self.get('model_number')
        else:
            raise IOError("ixaxis not recognised")

        pl.figure(ifig)
        pl.plot(xax,CO(self))

    def hrd(self,ifig=None,label=None,colour=None,s2ms=False):
        '''
        Plot an HR diagram
        
        Parameters
        ----------
        ifig : integer or string
            Figure label, if None the current figure is used
            The default value is None.
        lims : list [x_lower, x_upper, y_lower, y_upper]
        label : string
            Label for the model
            The default value is None
        colour : string
            The colour of the line
            The default value is None
        s2ms : boolean, optional
            "Skip to Main Sequence"?
            The default is False.
        '''
        
        fsize=18
        
        params = {'axes.labelsize':  fsize,
        #    'font.family':       'serif',
        'font.family':        'Times New Roman',
        'figure.facecolor':  'white',
        'text.fontsize':     fsize,
        'legend.fontsize':   fsize,
        'xtick.labelsize':   fsize*0.8,
        'ytick.labelsize':   fsize*0.8,
        'text.usetex':       False}
        pl.rcParams.update(params)
        
        if ifig is not None:
            pl.figure(ifig)
        
        if s2ms:
            h1=self.get('center_h1')
            idx=np.where(h1[0]-h1>=1.e-3)[0][0]
            skip=idx
        else:
            skip=0
        
        x = self.get('log_Teff')[skip:]
        y = self.get('log_L')[skip:]
        
        if label is not None:
            if colour is not None:
                pl.plot(x,y,label=label,color=colour)
            else:
                pl.plot(x,y,label=label)
            pl.legend(loc='best').draw_frame(False)
        else:
            if colour is not None:
                pl.plot(x,y,color=colour)
            else:
                pl.plot(x,y)

#        pyl.plot(self.data[:,self.cols['log_Teff']-1],\
#                 self.data[:,self.cols['log_L']-1],\
#                 label = "M="+str(self.header_attr['initial_mass'])+", Z="\
#                 +str(self.header_attr['initial_z']))

        pyl.xlabel('$\log T_{\\rm eff}$')
        pyl.ylabel('$\log L$')
        x1,x2=pl.xlim()
        if x2 > x1:
            ax=pl.gca()
            ax.invert_xaxis()
#            self._xlimrev()

    def hrd_key(self, key_str):
        ''' 
        plot an HR diagram

        Parameters
        ----------
        key_str : string
            A label string
            
        '''

        pyl.plot(self.data[:,self.cols['log_Teff']-1],\
                 self.data[:,self.cols['log_L']-1],label = key_str)
        pyl.legend()
        pyl.xlabel('log Teff')
        pyl.ylabel('log L')
        x1,x2=pl.xlim()
        if x2 > x1:
            self._xlimrev()

    def hrd_new(self, input_label="", skip=0):
        ''' 
        plot an HR diagram with options to skip the first N lines and
        add a label string 
        
        Parameters
        ----------
        input_label : string, optional
            Diagram label.  The default is "".
        skip : integer, optional
            Skip the first n lines.  The default is 0.
            
        '''
        xl_old=pyl.gca().get_xlim()
        if input_label == "":
            my_label="M="+str(self.header_attr['initial_mass'])+", Z="+str(self.header_attr['initial_z'])
        else:
            my_label="M="+str(self.header_attr['initial_mass'])+", Z="+str(self.header_attr['initial_z'])+"; "+str(input_label)

        pyl.plot(self.data[skip:,self.cols['log_Teff']-1],self.data[skip:,self.cols['log_L']-1],label = my_label)
        pyl.legend(loc=0)
        xl_new=pyl.gca().get_xlim()
        pyl.xlabel('log Teff')
        pyl.ylabel('log L')
        if any(array(xl_old)==0):
            pyl.gca().set_xlim(max(xl_new),min(xl_new))
        elif any(array(xl_new)==0):
            pyl.gca().set_xlim(max(xl_old),min(xl_old))
        else:
            pyl.gca().set_xlim([max(xl_old+xl_new),min(xl_old+xl_new)])

    def xche4_teff(self,ifig=None,lims=[1.,0.,3.4,4.7],label=None,colour=None,
                   s2ms=True):
        '''
        Plot effective temperature against central helium abundance.
        
        Parameters
        ----------
        ifig : integer or string
            Figure label, if None the current figure is used
            The default value is None.
        lims : list [x_lower, x_upper, y_lower, y_upper]
        label : string
        Label for the model
            The default value is None
        colour : string
            The colour of the line
            The default value is None
        s2ms : boolean, optional
            "Skip to Main Sequence"
            The default is True
        '''
        fsize=18
        params = {'axes.labelsize':  fsize,
        #    'font.family':       'serif',
        'font.family':        'Times New Roman',
        'figure.facecolor':  'white',
        'text.fontsize':     fsize,
        'legend.fontsize':   fsize,
        'xtick.labelsize':   fsize*0.8,
        'ytick.labelsize':   fsize*0.8,
        'text.usetex':       False}
        pl.rcParams.update(params)
        
        
        if s2ms:
            h1=self.get('center_h1')
            idx=np.where(h1[0]-h1>=1.e-3)[0][0]
            skip=idx
        else:
            skip=0

        x = self.get('center_he4')[skip:]
        y = self.get('log_Teff')[skip:]    
        if ifig is not None:
            pl.figure(ifig)
        if label is not None:
            if colour is not None:
                pl.plot(x,y,label=label,color=colour)
            else:
                pl.plot(x,y,label=label)
            pl.legend(loc='best').draw_frame(False)
        else:
            if colour is not None:
                pl.plot(x,y,color=colour)
            else:
                pl.plot(x,y)
        pl.xlim(lims[:2])
        pl.ylim(lims[2:])
        pl.xlabel('$X_{\\rm c}(\,^4{\\rm He}\,)$')
        pl.ylabel('$\log\,T_{\\rm eff}$')


    def tcrhoc(self,ifig=None,lims=[3.,10.,8.,10.],label=None,colour=None):
        '''
        Central temperature again central density plot
            
        Parameters
        ----------
        ifig : integer or string
            Figure label, if None the current figure is used
            The default value is None.
        lims : list [x_lower, x_upper, y_lower, y_upper]
        label : string
            Label for the model
            The default value is None
        colour : string
            The colour of the line
            The default value is None
        '''

        fsize=18

        params = {'axes.labelsize':  fsize,
            #    'font.family':       'serif',
        'font.family':        'Times New Roman',
        'figure.facecolor':  'white',
        'text.fontsize':     fsize,
        'legend.fontsize':   fsize,
        'xtick.labelsize':   fsize*0.8,
        'ytick.labelsize':   fsize*0.8,
        'text.usetex':       False}
        pl.rcParams.update(params)
        
        if ifig is not None:
            pl.figure(ifig)
        
        if label is not None:
            if colour is not None:
                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'),label=label,
                        color=colour)
            else:
                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'),label=label)
            pl.legend(loc='best').draw_frame(False)
        else:
            if colour is not None:
                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'),
                        color=colour)
            else:
                pl.plot(self.get('log_center_Rho'),self.get('log_center_T'))

        pl.xlim(lims[:2])
        pl.ylim(lims[2:])
        pl.xlabel('log $\\rho_{\\rm c}$')
        pl.ylabel('log $T_{\\rm c}$')

    def mdot_t(self,ifig=None,lims=[7.4,2.6,-8.5,-4.5],label=None,colour=None):
        '''
        Plot mass loss history as a function of log-time-left
        
        Parameters
        ----------
        ifig : integer or string
            Figure label, if None the current figure is used
            The default value is None.
        lims : list [x_lower, x_upper, y_lower, y_upper]
        label : string
            Label for the model
            The default value is None
        colour : string
            The colour of the line
            The default value is None
        '''
        
        fsize=18
        
        params = {'axes.labelsize':  fsize,
        #    'font.family':       'serif',
        'font.family':        'Times New Roman',
        'figure.facecolor':  'white',
        'text.fontsize':     fsize,
        'legend.fontsize':   fsize,
        'xtick.labelsize':   fsize*0.8,
        'ytick.labelsize':   fsize*0.8,
        'text.usetex':       False}
        pl.rcParams.update(params)
        
        if ifig is not None:
            pl.figure(ifig)
        
        gage= self.get('star_age')
        lage=np.zeros(len(gage))
        agemin = max(abs(gage[-1]-gage[-2])/5.,1.e-10)
        for i in np.arange(len(gage)):
            if gage[-1]-gage[i]>agemin:
                lage[i]=np.log10(gage[-1]-gage[i]+agemin)
            else :
                lage[i]=np.log10(agemin)
        x = lage
        y = self.get('log_abs_mdot')

        if ifig is not None:
            pl.figure(ifig)
        if label is not None:
            if colour is not None:
                pl.plot(x,y,label=label,color=colour)
            else:
                pl.plot(x,y,label=label)
            pl.legend(loc='best').draw_frame(False)
        else:
            if colour is not None:
                pl.plot(x,y,color=colour)
            else:
                pl.plot(x,y)
        pl.xlim(lims[:2])
        pl.ylim(lims[2:])
        pl.ylabel('$\mathrm{log}_{10}(\|\dot{M}\|/M_\odot\,\mathrm{yr}^{-1})$')
        pl.xlabel('$\mathrm{log}_{10}(t^*/\mathrm{yr})$')

    def mcc_t(self,ifig=None,lims=[0,15,0,25],label=None,colour=None,
              mask=False):
        '''
        Plot mass of convective core as a function of time.
        
        Parameters
        ----------
        ifig : integer or string
            Figure label, if None the current figure is used
            The default value is None.
        lims : list [x_lower, x_upper, y_lower, y_upper]
        label : string
            Label for the model
            The default value is None
        colour : string
            The colour of the line
            The default value is None
        mask : boolean, optional
            Do you want to try to hide numerical spikes in the
            plot?
            The default is False
        '''
        
        fsize=18
        
        params = {'axes.labelsize':  fsize,
        #    'font.family':       'serif',
        'font.family':        'Times New Roman',
        'figure.facecolor':  'white',
        'text.fontsize':     fsize,
        'legend.fontsize':   fsize,
        'xtick.labelsize':   fsize*0.8,
        'ytick.labelsize':   fsize*0.8,
        'text.usetex':       False}
        pl.rcParams.update(params)
        
        if ifig is not None:
            pl.figure(ifig)
        
        age= self.get('star_age')
        x1 = age / 1.e6
        x2 = age / 1.e6
        y1 = self.get('mix_qtop_1')*self.get('star_mass')
        y2 = self.get('mix_qtop_2')*self.get('star_mass')
        mt1 = self.get('mix_type_1')
        mt2 = self.get('mix_type_2')
        
        # Mask spikes...
        if mask:
            x1 = np.ma.masked_where(mt1 != 1, x1)
            x2 = np.ma.masked_where(mt2 != 1, x2)
            y1 = np.ma.masked_where(mt1 != 1, y1)
            y2 = np.ma.masked_where(mt2 != 1, y2)
        
        if ifig is not None:
            pl.figure(ifig)
        if label is not None:
            if colour is not None:
                pl.plot(x1,y1,label=label,color=colour)
                pl.plot(x2,y2,color=colour)
            else:
                pl.plot(x1,y1,label=label)
                pl.plot(x2,y2)
            pl.legend(loc='best').draw_frame(False)
        else:
            if colour is not None:
                pl.plot(x1,y1,color=colour)
                pl.plot(x2,y2,color=colour)
            else:
                pl.plot(x1,y1)
                pl.plot(x2,y2)
        pl.xlim(lims[:2])
        pl.ylim(lims[2:])
        pl.ylabel('$M/M_\odot}$')
        pl.xlabel('$t/{\\rm Myr}$')


    def kippenhahn_CO(self, num_frame, xax, t0_model=0,
                      title='Kippenhahn diagram', tp_agb=0.,
                      ylim_CO=[0,0]):
        ''' 
        Kippenhahn plot as a function of time or model with CO ratio

        Parameters
        ----------
        num_frame : integer
            Number of frame to plot this plot into.
        xax : string
            Either model or time to indicate what is to be used on the
            x-axis.
        t0_model : integer, optional
            Model for the zero point in time, for AGB plots this would
            be usually the model of the 1st TP, which can be found with
            the Kippenhahn plot.  The default is 0.
        title : string, optional
            Figure title.  The defalut is "Kippenhahn diagram".
        tp_agb : float, optional
            If >= 0 then,
            ylim=[h1_min*1.-tp_agb/100 : h1_max*1.+tp_agb/100] with
            h1_min, h1_max the min and max H-free core mass coordinate.
            The defalut is 0.
        ylim_CO : list
            if ylim_CO is [0,0], then it is automaticly set.  The
            default is [0,0].
            
        '''

        pyl.figure(num_frame)

        if xax == 'time':
            xaxisarray = self.get('star_age')
        elif xax == 'model':
            xaxisarray = self.get('model_number')
        else:
            print 'kippenhahn_error: invalid string for x-axis selction.'+\
                  ' needs to be "time" or "model"'

        t0_mod=xaxisarray[t0_model]

        plot_bounds=True
        try:
            h1_boundary_mass  = self.get('h1_boundary_mass')
            he4_boundary_mass = self.get('he4_boundary_mass')
        except:
            try:
                h1_boundary_mass  = self.get('he_core_mass')
                he4_boundary_mass = self.get('c_core_mass')
            except:
                plot_bounds=False

        star_mass         = self.get('star_mass')
        mx1_bot           = self.get('mx1_bot')*star_mass
        mx1_top           = self.get('mx1_top')*star_mass
        mx2_bot           = self.get('mx2_bot')*star_mass
        mx2_top           = self.get('mx2_top')*star_mass
        surface_c12       = self.get('surface_c12')
        surface_o16       = self.get('surface_o16')

        COratio=(surface_c12*4.)/(surface_o16*3.)

        pyl.plot(xaxisarray[t0_model:]-t0_mod,COratio[t0_model:],'-k',label='CO ratio')
        pyl.ylabel('C/O ratio')
        pyl.legend(loc=4)
        if ylim_CO[0] is not 0 and  ylim_CO[1] is not 0:
            pyl.ylim(ylim_CO)
        if xax == 'time':
            pyl.xlabel('t / yrs')
        elif xax == 'model':
            pyl.xlabel('model number')

        pyl.twinx()
        if plot_bounds:
            pyl.plot(xaxisarray[t0_model:]-t0_mod,h1_boundary_mass[t0_model:],label='h1_boundary_mass')
            pyl.plot(xaxisarray[t0_model:]-t0_mod,he4_boundary_mass[t0_model:],label='he4_boundary_mass')
        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx1_bot[t0_model:],',r',label='conv bound')
        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx1_top[t0_model:],',r')
        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx2_bot[t0_model:],',r')
        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx2_top[t0_model:],',r')
        pyl.plot(xaxisarray[t0_model:]-t0_mod,star_mass[t0_model:],label='star_mass')
        pyl.ylabel('mass coordinate')
        pyl.legend(loc=2)
        if tp_agb > 0.:
            h1_min = min(h1_boundary_mass[t0_model:])
            h1_max = max(h1_boundary_mass[t0_model:])
            h1_min = h1_min*(1.-tp_agb/100.)
            h1_max = h1_max*(1.+tp_agb/100.)
            print 'setting ylim to zoom in on H-burning:',h1_min,h1_max
            pyl.ylim(h1_min,h1_max)

    def kippenhahn(self, num_frame, xax, t0_model=0,
                   title='Kippenhahn diagram', tp_agb=0., t_eps=5.e2,
                   plot_star_mass=True, symbol_size=8, c12_bm=False,
                   print_legend=True):
        '''Kippenhahn plot as a function of time or model.

        Parameters
        ----------
        num_frame : integer
            Number of frame to plot this plot into, if <0 open no new
            figure.
        xax : string
            Either 'model', 'time' or 'logtimerev' to indicate what is
            to be used on the x-axis.
        t0_model : integer, optional
            If xax = 'time' then model for the zero point in time, for
            AGB plots this would be usually the model of the 1st TP,
            which can be found with the Kippenhahn plot.  The default
            is 0.
        title : string, optional
            The figure title.  The default is "Kippenhahn diagram".
        tp_agb : float, optional
            If > 0. then,
            ylim=[h1_min*1.-tp_agb/100 : h1_max*1.+tp_agb/100] with
            h1_min, h1_max the min and max H-free core mass coordinate.
            The default is 0. .
        t_eps : float, optional
            Final time for logtimerev.  The default is '5.e2'.
        plot_star_mass : boolean, optional
            If True, then plot the stellar mass as a line as well.  The
            default is True.
        symbol_size : integer, optional
            Size of convection boundary marker.  The default is 8.
        c12_bm : boolean, optional
            If we plot c12_boundary_mass or not.  The default is False.
        print_legend : boolean, optionla
            Show or do not show legend.  The defalut is True.

        '''

        if num_frame >= 0:
            pyl.figure(num_frame)

        t0_mod=[]

        if xax == 'time':
            xaxisarray = self.get('star_age')
            if t0_model > 0:
                ind=self.get('model_number')
                t0_model=where(ind>t0_model)[0][0]
                t0_mod=xaxisarray[t0_model]
            else:
                t0_mod = 0.
            print 'zero time is '+str(t0_mod)
        elif xax == 'model':
            xaxisarray = self.get('model_number')
            #t0_mod=xaxisarray[t0_model]
            t0_mod = 0.
        elif xax == 'logtimerev':
            xaxi    = self.get('star_age')
            xaxisarray = np.log10(np.max(xaxi)+t_eps-xaxi)
            t0_mod = 0.
        else:
            print 'kippenhahn_error: invalid string for x-axis selction.'+\
                  ' needs to be "time" or "model"'


        plot_bounds=True
        try:
            h1_boundary_mass  = self.get('h1_boundary_mass')
            he4_boundary_mass = self.get('he4_boundary_mass')
            if c12_bm:
                c12_boundary_mass = self.get('c12_boundary_mass')
        except:
            try:
                h1_boundary_mass  = self.get('he_core_mass')
                he4_boundary_mass = self.get('c_core_mass')
                if c12_bm:
                    c12_boundary_mass = self.get('o_core_mass')
            except:
                plot_bounds=False

        star_mass         = self.get('star_mass')
        mx1_bot           = self.get('mx1_bot')*star_mass
        mx1_top           = self.get('mx1_top')*star_mass
        mx2_bot           = self.get('mx2_bot')*star_mass
        mx2_top           = self.get('mx2_top')*star_mass


        if xax == 'time':
            if t0_model>0:
                pyl.xlabel('$t - t_0$ $\mathrm{[yr]}$')
            else:
                pyl.xlabel('t / yrs')
        elif xax == 'model':
            pyl.xlabel('model number')
        elif xax == 'logtimerev':
            pyl.xlabel('$\log(t_{final} - t)$  $\mathrm{[yr]}$')

        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx1_bot[t0_model:],linestyle='None',color='blue',alpha=0.3,marker='o',markersize=symbol_size,label='convection zones')
        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx1_top[t0_model:],linestyle='None',color='blue',alpha=0.3,marker='o',markersize=symbol_size)
        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx2_bot[t0_model:],linestyle='None',color='blue',alpha=0.3,marker='o',markersize=symbol_size)
        pyl.plot(xaxisarray[t0_model:]-t0_mod,mx2_top[t0_model:],linestyle='None',color='blue',alpha=0.3,marker='o',markersize=symbol_size)

        if plot_bounds:
            pyl.plot(xaxisarray[t0_model:]-t0_mod,h1_boundary_mass[t0_model:],color='red',linewidth=2,label='H-free core')
            pyl.plot(xaxisarray[t0_model:]-t0_mod,he4_boundary_mass[t0_model:],color='green',linewidth=2,linestyle='dashed',label='He-free core')
            if c12_bm:
                pyl.plot(xaxisarray[t0_model:]-t0_mod,c12_boundary_mass[t0_model:],color='purple',linewidth=2,linestyle='dotted',label='C-free core')
        if plot_star_mass is True:
            pyl.plot(xaxisarray[t0_model:]-t0_mod,star_mass[t0_model:],label='$M_\star$')
        pyl.ylabel('$m_\mathrm{r}/\mathrm{M}_\odot$')
        if print_legend:
            pyl.legend(loc=2)
        if tp_agb > 0.:
            h1_min = min(h1_boundary_mass[t0_model:])
            h1_max = max(h1_boundary_mass[t0_model:])
            h1_min = h1_min*(1.-tp_agb/100.)
            h1_max = h1_max*(1.+tp_agb/100.)
            print 'setting ylim to zoom in on H-burning:',h1_min,h1_max
            pyl.ylim(h1_min,h1_max)

    def t_surfabu(self, num_frame, xax, t0_model=0,
                  title='surface abundance', t_eps=1.e-3,
                  plot_CO_ratio=False):
        ''' 
        t_surfabu plots surface abundance evolution as a function of
        time.
        
        Parameters
        ----------
        num_frame : integer
            Number of frame to plot this plot into, if <0 don't open
            figure.
        xax : string
            Either model, time or logrevtime to indicate what is to be
            used on the x-axis.
        t0_model : integer, optional
            Model for the zero point in time, for AGB plots this would
            be usually the model of the 1st TP, which can be found with
            the Kippenhahn plot.  The default is 0.
        title : string, optional
            Figure title.  The default is "surface abundance".
        t_eps : float, optional
            Time eps at end for logrevtime.  The default is 1.e-3.
        plot_CO_ratio : boolean, optional
            On second axis True/False.  The default is False.

        '''
        if num_frame >= 0:
            pyl.figure(num_frame)

        if xax == 'time':
            xaxisarray = self.get('star_age')[t0_model:]
        elif xax == 'model':
            xaxisarray = self.get('model_number')[t0_model:]
        elif xax == 'logrevtime':
            xaxisarray = self.get('star_age')
            xaxisarray=np.log10(max(xaxisarray[t0_model:])+t_eps-xaxisarray[t0_model:])
        else:
            print 't-surfabu error: invalid string for x-axis selction.'+ \
                  ' needs to be "time" or "model"'

        star_mass         = self.get('star_mass')
        surface_c12       = self.get('surface_c12')
        surface_c13       = self.get('surface_c13')
        surface_n14       = self.get('surface_n14')
        surface_o16       = self.get('surface_o16')

        target_n14 = -3.5


        COratio=(surface_c12*4.)/(surface_o16*3.)
        t0_mod=xaxisarray[t0_model]
        log10_c12=np.log10(surface_c12[t0_model:])

        symbs=['k:','-','--','-.','b:','-','--','k-.',':','-','--','-.']

        pyl.plot(xaxisarray,log10_c12,\
                     symbs[0],label='$^{12}\mathrm{C}$')
        pyl.plot(xaxisarray,np.log10(surface_c13[t0_model:]),\
                     symbs[1],label='$^{13}\mathrm{C}$')
        pyl.plot(xaxisarray,np.log10(surface_n14[t0_model:]),\
                     symbs[2],label='$^{14}\mathrm{N}$')
        pyl.plot(xaxisarray,np.log10(surface_o16[t0_model:]),\
                     symbs[3],label='$^{16}\mathrm{O}$')
#                pyl.plot([min(xaxisarray[t0_model:]-t0_mod),max(xaxisarray[t0_model:]-t0_mod)],[target_n14,target_n14])

        pyl.ylabel('mass fraction $\log X$')
        pyl.legend(loc=2)

        if xax == 'time':
            pyl.xlabel('t / yrs')
        elif xax == 'model':
            pyl.xlabel('model number')
        elif xax == 'logrevtime':
            pyl.xlabel('$\\log t-tfinal$')
        if plot_CO_ratio:
            pyl.twinx()
            pyl.plot(xaxisarray,COratio[t0_model:],'-k',label='CO ratio')
            pyl.ylabel('C/O ratio')
            pyl.legend(loc=4)
        pyl.title(title)
        if xax == 'logrevtime':
            self._xlimrev()


# ... end t_surfabu

    def t_lumi(self,num_frame,xax):
        ''' 
        Luminosity evolution as a function of time or model.

        Parameters
        ----------
        num_frame : integer
            Number of frame to plot this plot into.
        xax : string
            Either model or time to indicate what is to be used on the
            x-axis

        '''

        pyl.figure(num_frame)

        if xax == 'time':
            xaxisarray = self.get('star_age')
        elif xax == 'model':
            xaxisarray = self.get('model_number')
        else:
            print 'kippenhahn_error: invalid string for x-axis selction. needs to be "time" or "model"'


        logLH   = self.get('log_LH')
        logLHe  = self.get('log_LHe')

        pyl.plot(xaxisarray,logLH,label='L_(H)')
        pyl.plot(xaxisarray,logLHe,label='L(He)')
        pyl.ylabel('log L')
        pyl.legend(loc=2)


        if xax == 'time':
            pyl.xlabel('t / yrs')
        elif xax == 'model':
            pyl.xlabel('model number')

    def t_surf_parameter(self, num_frame, xax):
        ''' 
        Surface parameter evolution as a function of time or model.

        Parameters
        ----------
        num_frame : integer
            Number of frame to plot this plot into.
        xax : string
            Either model or time to indicate what is to be used on the
            x-axis

        '''

        pyl.figure(num_frame)

        if xax == 'time':
            xaxisarray = self.get('star_age')
        elif xax == 'model':
            xaxisarray = self.get('model_number')
        else:
            print 'kippenhahn_error: invalid string for x-axis selction. needs to be "time" or "model"'


        logL    = self.get('log_L')
        logTeff    = self.get('log_Teff')

        pyl.plot(xaxisarray,logL,'-k',label='log L')
        pyl.plot(xaxisarray,logTeff,'-k',label='log Teff')
        pyl.ylabel('log L, log Teff')
        pyl.legend(loc=2)


        if xax == 'time':
            pyl.xlabel('t / yrs')
        elif xax == 'model':
            pyl.xlabel('model number')

    def _kip_vline(self, modstart, modstop, sparse, outfile,
                  xlims=[0.,0.], ylims=[0.,0.], ixaxis='log_time_left',
                  mix_zones=5, burn_zones=50):
        '''
        *** DEPRECIATED and hence UNSUPPORTED ***
        This function creates a Kippenhahn plot with energy flux using
        vertical lines, better thermal pulse resolution.
        
        For a more comprehensive plot, your history.data or star.log
        file should contain columns called "mix_type_n", "mix_qtop_n", 
        "burn_type_n" and "burn_qtop_n".  The number of columns
        (i.e. the bbiggest value of n) is what goes in the arguments as
        mix_zones and burn_zones.
        
        DO NOT WORRY! if you do not have these columns, just leave the
        default values alone and the script should recognise that you
        do not have these columns and make the most detailed plot that
        is available to you.

        Parameters
        ----------
        modstart : integer
            Model from which you want to plot (be careful if your
            history.data or star.log output is sparse...).
        modstop : integer
            Model to which you wish to plot.
        sparse : integer
            x-axis sparsity.
        outfile : string
            'filename + extension' where you want to save the figure.
        xlims, ylims : list, optional
            plot limits, however these are somewhat obsolete now that
            we have modstart and modstop.  Leaving them as 0. is
            probably no slower, and you can always zoom in afterwards
            in mpl.  The default is [0., 0.,].
        ixaxis : string, optional
            Either 'log_time_left', 'age', or 'model_number'.  The
            default "log_time_left".
        mix_zones, burn_zones : integer
            As described above, if you have more detailed output about
            your convection and energy generation boundaries in columns
            mix_type_n, mix_qtop_n, burn_type_n and burn_qtop_n, you
            need to specify the total number of columns for mixing zones
            and burning zones that you have.  Can't work this out from
            your history.data or star.log file?  Check the
            history_columns.list that you used, it'll be the number
            after "mixing regions" and "burning regions".  Can't see
            these columns?  leave it and 2 conv zones and 2 burn zones
            will be drawn using other data that you certainly should
            have in your history.data or star.log file.  The default for
            mix_zones is 5, the defalut for burn_zones is 50.
            
        '''


        xxyy=[self.get('star_age')[modstart:modstop],self.get('star_age')[modstart:modstop]]
        mup = max(float(self.get('star_mass')[0])*1.02,1.0)
        nmodels=len(self.get('model_number')[modstart:modstop])

        Msol=1.98892E+33

        engenstyle = 'full'

        dx = sparse
        x = np.arange(0, nmodels, dx)

        btypemax = 20
        btypemin = -20
        btypealpha=0.

        ########################################################################
        #----------------------------------plot--------------------------------#
        fig = pl.figure()
#       fig.set_size_inches(16,9)
        fsize=15
        ax=pl.axes()

        if ixaxis == 'log_time_left':
        # log of time left until core collapse
            gage= self.get('star_age')
            lage=np.zeros(len(gage))
            agemin = max(abs(gage[-1]-gage[-2])/5.,1.e-10)
            for i in np.arange(len(gage)):
                if gage[-1]-gage[i]>agemin:
                    lage[i]=np.log10(gage[-1]-gage[i]+agemin)
                else :
                    lage[i]=np.log10(agemin)
            xxx = lage[modstart:modstop]
            print 'plot versus time left'
            ax.set_xlabel('$\mathrm{log}_{10}(t^*) \, \mathrm{(yr)}$',fontsize=fsize)
        elif ixaxis =='model_number':
            xxx= self.get('model_number')[modstart:modstop]
            print 'plot versus model number'
            ax.set_xlabel('Model number',fontsize=fsize)
        elif ixaxis =='age':
            xxx= self.get('star_age')[modstart:modstop]/1.e6
            print 'plot versus age'
            ax.set_xlabel('Age [Myr]',fontsize=fsize)
        else:
            print 'ixaxis must be one of: log_time_left, age or model_number'
            sys.exit()

        if xlims == [0.,0.]:
            xlims[0] = xxx[0]
            xlims[1] = xxx[-1]
        if ylims == [0.,0.]:
            ylims[0] = 0.
            ylims[1] = mup


        print 'plotting patches'
        ax.plot(xxx[::dx],self.get('star_mass')[modstart:modstop][::dx],'k-')

        print 'plotting abund boundaries'
        ax.plot(xxx,self.get('h1_boundary_mass')[modstart:modstop],label='H boundary')
        ax.plot(xxx,self.get('he4_boundary_mass')[modstart:modstop],label='He boundary')
#       ax.plot(xxx,self.get('c12_boundary_mass')[modstart:modstop],label='C boundary')

        ax.axis([xlims[0],xlims[1],ylims[0],ylims[1]])

        ax.set_ylabel('Mass [M$_\odot$]')

        ########################################################################

        try:
            self.get('burn_qtop_1')
        except:
            engenstyle = 'twozone'
        if engenstyle == 'full':
            for i in range(len(x)):
                # writing reading status
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map1 " + "...%d%%" % percent)
                for j in range(1,burn_zones+1):
                    ulimit=self.get('burn_qtop_'+str(j))[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                    if j==1:
                        llimit=0.0
                    else:
                        llimit=self.get('burn_qtop_'+str(j-1))[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                    btype=float(self.get('burn_type_'+str(j))[modstart:modstop][i*dx])
                    if llimit!=ulimit:
                        if btype>0.:
                            #btypealpha = btype/btypemax
                            #ax.axvline(xxx[i*dx],ymin=(llimit-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimit-ylims[0])/(ylims[1]-ylims[0]),color='b',alpha=btypealpha)
                            pass
                        if btype<0.:
                            #btypealpha = (btype/btypemin)/5
                            #ax.axvline(xxx[i*dx],ymin=(llimit-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimit-ylims[0])/(ylims[1]-ylims[0]),color='r',alpha=btypealpha)
                            pass

        if engenstyle == 'twozone':
            for i in range(len(x)):
            # writing reading status
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map1 " + "...%d%%" % percent)
                llimitl1=self.get('epsnuc_M_1')[modstart:modstop][i*dx]/Msol
                ulimitl1=self.get('epsnuc_M_4')[modstart:modstop][i*dx]/Msol
                llimitl2=self.get('epsnuc_M_5')[modstart:modstop][i*dx]/Msol
                ulimitl2=self.get('epsnuc_M_8')[modstart:modstop][i*dx]/Msol
                llimith1=self.get('epsnuc_M_2')[modstart:modstop][i*dx]/Msol
                ulimith1=self.get('epsnuc_M_3')[modstart:modstop][i*dx]/Msol
                llimith2=self.get('epsnuc_M_6')[modstart:modstop][i*dx]/Msol
                ulimith2=self.get('epsnuc_M_7')[modstart:modstop][i*dx]/Msol
                # lower thresh first, then upper thresh:
                #if llimitl1!=ulimitl1:
                    #ax.axvline(xxx[i*dx],ymin=(llimitl1-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimitl1-ylims[0])/(ylims[1]-ylims[0]),color='b',alpha=1.)
                #if llimitl2!=ulimitl2:
                    #ax.axvline(xxx[i*dx],ymin=(llimitl2-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimitl2-ylims[0])/(ylims[1]-ylims[0]),color='b',alpha=1.)
                #if llimith1!=ulimith1:
                    #ax.axvline(xxx[i*dx],ymin=(llimith1-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimith1-ylims[0])/(ylims[1]-ylims[0]),color='b',alpha=4.)
                #if llimith2!=ulimith2:
                    #ax.axvline(xxx[i*dx],ymin=(llimith2-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimith2-ylims[0])/(ylims[1]-ylims[0]),color='b',alpha=4.)

        mixstyle = 'full'
        try:
            self.get('mix_qtop_1')
        except:
            mixstyle = 'twozone'
        if mixstyle == 'full':
            for i in range(len(x)):
            # writing reading status
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map2 " + "...%d%%" % percent)
                for j in range(1,mix_zones+1):
                    ulimit=self.get('mix_qtop_'+str(j))[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                    if j==1:
                        llimit=0.0
                    else:
                        llimit=self.get('mix_qtop_'+str(j-1))[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                    mtype=self.get('mix_type_'+str(j))[modstart:modstop][i*dx]
                    if llimit!=ulimit:
                        if mtype == 1:
                            ax.axvline(xxx[i*dx],ymin=(llimit-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimit-ylims[0])/(ylims[1]-ylims[0]),color='k',alpha=3., linewidth=.5)
        if mixstyle == 'twozone':
            for i in range(len(x)):
            # writing reading status
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map2 " + "...%d%%" % percent)
                ulimit=self.get('conv_mx1_top')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                llimit=self.get('conv_mx1_bot')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                if llimit!=ulimit:
                    ax.axvline(xxx[i*dx],ymin=(llimit-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimit-ylims[0])/(ylims[1]-ylims[0]),color='k',alpha=5.,linewidth=.5)
                ulimit=self.get('conv_mx2_top')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                llimit=self.get('conv_mx2_bot')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                if llimit!=ulimit:
                    ax.axvline(xxx[i*dx],ymin=(llimit-ylims[0])/(ylims[1]-ylims[0]),ymax=(ulimit-ylims[0])/(ylims[1]-ylims[0]),color='k',alpha=3.,linewidth=.5)

        print 'engenstyle was ', engenstyle
        print 'mixstyle was ', mixstyle
        print '\n finished preparing color map'

        #fig.savefig(outfile)
        pl.show()

    def kip_cont(self, ifig=110, modstart=0, modstop=-1,t0_model=0,
                 outfile='out.png', xlims=[0.,0.], ylims=[0.,0.],
                 xres=1000, yres=1000, ixaxis='model_number',
                 mix_zones=20, burn_zones=20, plot_radius=False,
                 engenPlus=True, engenMinus=False,
                 landscape_plot=False, rad_lines=False, profiles=[],
                 showfig=True, outlines=True, boundaries=True,
                 c12_boundary=False, rasterise=False, yscale='1.',
                 engenlevels=None):
        ''' 
        This function creates a Kippenhahn plot with energy flux using
        contours.

        This plot uses mixing_regions and burning_regions written to
        your history.data or star.log. Set both variables in the
        log_columns.list file to 20 as a start.

        The output log file should then contain columns called
        "mix_type_n", "mix_qtop_n", "burn_type_n" and "burn_qtop_n".
        The number of columns (i.e. the bbiggest value of n) is what
        goes in the arguments as mix_zones and burn_zones.  DO NOT
        WORRY! if you do not have these columns, just leave the default
        values alone and the script should recognise that you do not
        have these columns and make the most detailed plot that is
        available to you.

        Defaults are set to get some plot, that may not look great if
        you zoom in interactively. Play with xres and yres as well as
        setting the xlims to ylims to the region you are interested in.

        Parameters
        ----------
        ifig : integer, optional
            Figure frame number.  The default is 110.
        modstart : integer, optional
            Model from which you want to plot (be careful if your
            history.data or star.log output is sparse...).  If it is
            0 then it starts from the beginning, works even if
            log_cnt > 1.
            The default is 0.
        modstop : integer, optional
            Model to which you wish to plot, -1 corresponds to end
            [if log_cnt>1, devide modstart and modstop by log_cnt,
            this needs to be improved! SJ: this should be ficed now].
            The defalut is -1.
        t0_model : integer, optional
            Model number from which to reset the time to 0. Typically,
            if modstart!=0, t0_model=modstart is a good choice, but we
            leave the choice to the user in case the time is wished to
            start from 0 at a different key point of the evolution.
            The default value is 0.
        outfile : sting, optional
            'filename + extension' where you want to save the figure.
            The defalut is "out.png".
        xlims, ylims : list, optional
            Plot limits, however these are somewhat obsolete now that
            we have modstart and modstop.  Leaving them as 0. is
            probably no slower, and you can always zoom in afterwards
            in mpl.  ylims is important for well resolved thermal
            pulse etc plots; it's best to get the upper and lower limits
            of he-intershell using s.kippenhahn_CO(1,'model') first.
            The default is [0., 0.].
        xres, yres : integer, optional
            plot resolution. Needless to say that increasing these
            values will yield a nicer plot with some slow-down in
            plotting time.  You will most commonly change xres.  For a
            prelim plot, try xres~200, then bump it up to anywhere from
            1000-10000 for real nicely resolved, publication quality
            plots.  The default is 1000.
        ixaxis : string, optional
            Either 'log_time_left', 'age', or 'model_number'.  The
            default is "model_number".
        mix_zones, burn_zones : integer, optional
            As described above, if you have more detailed output about
            your convection and energy generation boundaries in columns
            mix_type_n, mix_qtop_n, burn_type_n and burn_qtop_n, you
            need to specify the total number of columns for mixing zones
            and burning zones that you have.  Can't work this out from
            your history.data or star.log file?  Check the
            history_columns.list that you used, it'll be the number
            after "mixing regions" and "burning regions".  Can't see
            these columns?  leave it and 2 conv zones and 2 burn zones
            will be drawn using other data that you certainly should
            have in your history.data or star.log file.  The defalut for
            both is 20.
        plot_radius : boolean, optional
            Whether on a second y-axis you want to plot the radius of
            the surface and the he-free core.  The default is False.
        engenPlus : boolean
            Plot energy generation contours for eps_nuc>0.  The default
            is True.
        endgenMinus : boolean, optional
            Plot energy generation contours for eos_nuc<0.  The default
            is True.
        landscape_plot : boolean, optionla
            The default is False.
        rad_lines : boolean, optional
            The deafault is False.
        profiles : list, optional
            The default is [].
        showfig : boolean, optional
            The default is True.
        outlines : boolean, optional
            Whether or not to plot outlines of conv zones in darker
            colour.
        boundaries : boolean, optional
            Whether or not to plot H-, He- and C-free boundaries.
        c12_boundary : boolean, optional
            The default is False.
        rasterise : boolean, optional
            Whether or not to rasterise the contour regions to make
            smaller vector graphics figures.  The default is False.
        yscale : string, optional
            Re-scale the y-axis by this amount
        engenlevels : list
            Give cusstom levels to the engenPlus contour. If None,
            the levels are chosen automatically.
            The default is None.
            
        Notes
        -----
        The parameter xlims is depricated.
        
        '''
        
        # Find correct modstart and modstop:
        mod=np.array([int(i) for i in self.get('model_number')])
        mod1=np.abs(mod-modstart).argmin()
        mod2=np.abs(mod-modstop).argmin()
        if modstart != 0 : modstart=mod1
        if modstop != -1 : modstop=mod2
        
        xxyy=[self.get('star_age')[modstart:modstop],self.get('star_age')[modstart:modstop]]
        mup = max(float(self.get('star_mass')[0])*1.02,1.0)
        nmodels=len(self.get('model_number')[modstart:modstop])

        if ylims == [0.,0.]:
            mup   = max(float(self.get('star_mass')[0])*1.02,1.0)
            mDOWN = 0.
        else:
            mup = ylims[1]
            mDOWN = ylims[0]

        # y-axis resolution
        ny=yres
        #dy=mup/float(ny)
        dy = (mup-mDOWN)/float(ny)

        # x-axis resolution
        maxpoints=xres
        dx=int(max(1,nmodels/maxpoints))

        #y = np.arange(0., mup, dy)
        y = np.arange(mDOWN, mup, dy)
        x = np.arange(0, nmodels, dx)
        Msol=1.98892E+33

        engenstyle = 'full'

        B1=np.zeros([len(y),len(x)],float)
        B2=np.zeros([len(y),len(x)],float)
        try:
            self.get('burn_qtop_1')
        except:
            engenstyle = 'twozone'
        if engenstyle == 'full' and (engenPlus == True or engenMinus == True):
            ulimit_array = np.array([self.get('burn_qtop_'+str(j))[modstart:modstop:dx]*self.get('star_mass')[modstart:modstop:dx] for j in range(1,burn_zones+1)])
            #ulimit_array = np.around(ulimit_array,decimals=len(str(dy))-2)
            llimit_array = np.delete(ulimit_array,-1,0)
            llimit_array = np.insert(ulimit_array,0,0.,0)
            #llimit_array = np.around(llimit_array,decimals=len(str(dy))-2)
            btype_array = np.array([self.get('burn_type_'+str(j))[modstart:modstop:dx] for j in range(1,burn_zones+1)])
            for i in range(len(x)):
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map burn " + "...%d%%" % percent)
                for j in range(burn_zones):
                    if btype_array[j,i] > 0. and abs(btype_array[j,i]) < 99.:
                        B1[(np.abs(y-llimit_array[j][i])).argmin():(np.abs(y-ulimit_array[j][i])).argmin()+1,i] = 10.0**(btype_array[j,i])
                    elif btype_array[j,i] < 0. and abs(btype_array[j,i]) < 99.:
                        B2[(np.abs(y-llimit_array[j][i])).argmin():(np.abs(y-ulimit_array[j][i])).argmin()+1,i] = 10.0**(abs(btype_array[j,i]))

        if engenstyle == 'twozone' and (engenPlus == True or engenMinus == True):
            V=np.zeros([len(y),len(x)],float)
            for i in range(len(x)):
            # writing reading status
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map1 " + "...%d%%" % percent)
                llimitl1=self.get('epsnuc_M_1')[modstart:modstop][i*dx]/Msol
                ulimitl1=self.get('epsnuc_M_4')[modstart:modstop][i*dx]/Msol
                llimitl2=self.get('epsnuc_M_5')[modstart:modstop][i*dx]/Msol
                ulimitl2=self.get('epsnuc_M_8')[modstart:modstop][i*dx]/Msol
                llimith1=self.get('epsnuc_M_2')[modstart:modstop][i*dx]/Msol
                ulimith1=self.get('epsnuc_M_3')[modstart:modstop][i*dx]/Msol
                llimith2=self.get('epsnuc_M_6')[modstart:modstop][i*dx]/Msol
                ulimith2=self.get('epsnuc_M_7')[modstart:modstop][i*dx]/Msol
                # lower thresh first, then upper thresh:
                if llimitl1!=ulimitl1:
                    for k in range(ny):
                        if llimitl1<=y[k] and ulimitl1>y[k]:
                            V[k,i]=10.
                if llimitl2!=ulimitl2:
                    for k in range(ny):
                        if llimitl2<=y[k] and ulimitl2>y[k]:
                            V[k,i]=10.
                if llimith1!=ulimith1:
                    for k in range(ny):
                        if llimith1<=y[k] and ulimith1>y[k]:
                            V[k,i]=30.
                if llimith2!=ulimith2:
                    for k in range(ny):
                        if llimith2<=y[k] and ulimith2>y[k]:
                            V[k,i]=30.

        mixstyle = 'full'
        try:
            self.get('mix_qtop_1')
        except:
            mixstyle = 'twozone'
        if mixstyle == 'full':
            Z=np.zeros([len(y),len(x)],float)
            ulimit_array = np.array([self.get('mix_qtop_'+str(j))[modstart:modstop:dx]*self.get('star_mass')[modstart:modstop:dx] for j in range(1,mix_zones+1)])
            llimit_array = np.delete(ulimit_array,-1,0)
            llimit_array = np.insert(ulimit_array,0,0.,0)
            mtype_array = np.array([self.get('mix_type_'+str(j))[modstart:modstop:dx] for j in range(1,mix_zones+1)])
            for i in range(len(x)):
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map mix " + "...%d%%" % percent)
                for j in range(mix_zones):
                    if mtype_array[j,i] == 1.:
                        Z[(np.abs(y-llimit_array[j][i])).argmin():(np.abs(y-ulimit_array[j][i])).argmin()+1,i] = 1.

        if mixstyle == 'twozone':
            Z=np.zeros([len(y),len(x)],float)
            for i in range(len(x)):
            # writing reading status
                percent = int(i*100/len(x))
                sys.stdout.flush()
                sys.stdout.write("\rcreating color map2 " + "...%d%%" % percent)
                ulimit=self.get('conv_mx1_top')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                llimit=self.get('conv_mx1_bot')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                if llimit!=ulimit:
                    for k in range(ny):
                        if llimit<=y[k] and ulimit>y[k]:
                            Z[k,i]=1.
                ulimit=self.get('conv_mx2_top')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                llimit=self.get('conv_mx2_bot')[modstart:modstop][i*dx]*self.get('star_mass')[modstart:modstop][i*dx]
                if llimit!=ulimit:
                    for k in range(ny):
                        if llimit<=y[k] and ulimit>y[k]:
                            Z[k,i]=1.

        if rad_lines == True:
            masses = np.arange(0.1,1.5,0.1)
            rads=[[],[],[],[],[],[],[],[],[],[],[],[],[],[]]
            modno=[]
            for i in range(len(profiles)):
                p=mesa_profile('./LOGS',profiles[i])
                modno.append(p.header_attr['model_number'])
                for j in range(len(masses)):
                    idx=np.abs(p.get('mass')-masses[j]).argmin()
                    rads[j].append(p.get('radius')[idx])

        print 'engenstyle was ', engenstyle
        print 'mixstyle was ', mixstyle
        print '\n finished preparing color map'

        ########################################################################
        #----------------------------------plot--------------------------------#
        fig = pyl.figure(ifig)
        fsize=20
        if landscape_plot == True:
            fig.set_size_inches(9,4)
            pl.gcf().subplots_adjust(bottom=0.2)
            pl.gcf().subplots_adjust(right=0.85)
        params = {'axes.labelsize':  fsize,
          'text.fontsize':   fsize,
          'legend.fontsize': fsize,
          'xtick.labelsize': fsize*0.8,
          'ytick.labelsize': fsize*0.8,
          'text.usetex': False}
        pyl.rcParams.update(params)

        #ax=pl.axes([0.1,0.1,0.9,0.8])

        #fig=pl.figure()
        ax=pl.axes()

        if ixaxis == 'log_time_left':
        # log of time left until core collapse
            gage= self.get('star_age')
            lage=np.zeros(len(gage))
            agemin = max(abs(gage[-1]-gage[-2])/5.,1.e-10)
            for i in np.arange(len(gage)):
                if gage[-1]-gage[i]>agemin:
                    lage[i]=np.log10(gage[-1]-gage[i]+agemin)
                else :
                    lage[i]=np.log10(agemin)
            xxx = lage[modstart:modstop]
            print 'plot versus time left'
            ax.set_xlabel('$\mathrm{log}_{10}(t^*) \, \mathrm{(yr)}$',fontsize=fsize)
            if xlims[1] == 0.:
                xlims = [xxx[0],xxx[-1]]
        elif ixaxis =='model_number':
            xxx= self.get('model_number')[modstart:modstop]
            print 'plot versus model number'
            ax.set_xlabel('Model number',fontsize=fsize)
            if xlims[1] == 0.:
                xlims = [self.get('model_number')[modstart],self.get('model_number')[modstop]]
        elif ixaxis =='age':
            if t0_model != 0:
                t0_mod=np.abs(mod-t0_model).argmin()
                xxx= self.get('star_age')[modstart:modstop] - self.get('star_age')[t0_mod]
                print 'plot versus age'
                ax.set_xlabel('Age [yr] - '+str(self.get('star_age')[modstart]),fontsize=fsize)
            else:
                xxx= self.get('star_age')[modstart:modstop]/1.e6
                ax.set_xlabel('Age [Myr]',fontsize=fsize)
            if xlims[1] == 0.:
                xlims = [xxx[0],xxx[-1]]

        ax.set_ylabel('$\mathrm{Mass }(M_\odot)$')

        # some stuff for rasterizing only the contour part of the plot, for nice, but light, eps:
        class ListCollection(Collection):
            def __init__(self, collections, **kwargs):
                Collection.__init__(self, **kwargs)
                self.set_collections(collections)
            def set_collections(self, collections):
                self._collections = collections
            def get_collections(self):
                return self._collections
            @allow_rasterization
            def draw(self, renderer):
                for _c in self._collections:
                    _c.draw(renderer)

        def insert_rasterized_contour_plot(c):
            collections = c.collections
            for _c in collections:
                _c.remove()
            cc = ListCollection(collections, rasterized=True)
            ax = pl.gca()
            ax.add_artist(cc)
            return cc

        cmapMIX = matplotlib.colors.ListedColormap(['w','#8B8386']) # rose grey
        cmapB1  = pyl.cm.get_cmap('Blues')
        cmapB2  = pl.cm.get_cmap('Reds')
        
        ylims1=[0.,0.]
        ylims1[0]=ylims[0]
        ylims1[1]=ylims[1]
        if ylims == [0.,0.]:
            ylims[0] = 0.
            ylims[1] = mup
        if ylims[0] != 0.:
            ylab='$(\mathrm{Mass }$ - '+str(ylims[0])
            if yscale!='1.':
                ylab+=') / '+yscale+' $M_\odot$'
            else:
                ylab+=') / $M_\odot$'
            ax.set_ylabel(ylab)
            y = y - ylims[0]
            y = y*float(yscale) # SJONES tweak
            ylims[0] = y[0]
            ylims[1] = y[-1]

        print 'plotting contours'
        CMIX    = ax.contourf(xxx[::dx],y,Z, cmap=cmapMIX,alpha=0.6,levels=[0.5,1.5])
        if rasterise==True:
            insert_rasterized_contour_plot(CMIX)
        if outlines == True:
            CMIX_outlines    = ax.contour(xxx[::dx],y,Z, cmap=cmapMIX)
            if rasterise==True:
                insert_rasterized_contour_plot(CMIX_outlines)

        if engenstyle == 'full' and engenPlus == True:
            if engenlevels!= None:
                CBURN1  = ax.contourf(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.5, locator=matplotlib.ticker.LogLocator(),levels=engenlevels)
                if outlines:
                    CB1_outlines  = ax.contour(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.7, locator=matplotlib.ticker.LogLocator(),levels=engenlevels)
            else:
                CBURN1  = ax.contourf(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.5, locator=matplotlib.ticker.LogLocator())
                if outlines:
                    CB1_outlines  = ax.contour(xxx[::dx],y,B1, cmap=cmapB1, alpha=0.7, locator=matplotlib.ticker.LogLocator())
            CBARBURN1 = pyl.colorbar(CBURN1)
            CBARBURN1.set_label('$|\epsilon_\mathrm{nuc}-\epsilon_{\\nu}| \; (\mathrm{erg\,g}^{-1}\mathrm{\,s}^{-1})$',fontsize=fsize)
            if rasterise==True:
                insert_rasterized_contour_plot(CBURN1)
                if outlines:
                    insert_rasterized_contour_plot(CB1_outlines)

        if engenstyle == 'full' and engenMinus == True:
            CBURN2  = ax.contourf(xxx[::dx],y,B2, cmap=cmapB2, alpha=0.5, locator=matplotlib.ticker.LogLocator())
            if outlines:
                CBURN2_outlines  = ax.contour(xxx[::dx],y,B2, cmap=cmapB2, alpha=0.7, locator=matplotlib.ticker.LogLocator())
            CBARBURN2 = pl.colorbar(CBURN2)
            if engenPlus == False:
                CBARBURN2.set_label('$|\epsilon_\mathrm{nuc}-\epsilon_{\\nu}| \; (\mathrm{erg\,g}^{-1}\mathrm{\,s}^{-1})$',fontsize=fsize)
            if rasterise==True:
                insert_rasterized_contour_plot(CBURN2)
                if outlines:
                    insert_rasterized_contour_plot(CB2_outlines)

        if engenstyle == 'twozone' and (engenPlus == True or engenMinus == True):
            ax.contourf(xxx[::dx],y,V, cmap=cmapB1, alpha=0.5)

        print 'plotting patches'
        mtot=self.get('star_mass')[modstart:modstop][::dx]
        mtot1=(mtot-ylims1[0])*float(yscale)
        ax.plot(xxx[::dx],mtot,'k-')

        if boundaries == True:
            print 'plotting abund boundaries'
            try:
                bound=self.get('h1_boundary_mass')[modstart:modstop]
                bound1=(bound-ylims1[0])*float(yscale)
                ax.plot(xxx,bound1,label='H boundary',linestyle='-')
                
                bound=self.get('he4_boundary_mass')[modstart:modstop]
                bound1=(bound-ylims1[0])*float(yscale)
                ax.plot(xxx,bound1,label='He boundary',linestyle='--')
                
                bound=self.get('c12_boundary_mass')[modstart:modstop]
                bound1=(bound-ylims1[0])*float(yscale)
                ax.plot(xxx,bound1,label='C boundary',linestyle='-.')

            except:
                try:
                    bound=self.get('he_core_mass')[modstart:modstop]
                    bound1=(bound-ylims1[0])*float(yscale)
                    ax.plot(xxx,bound1,label='H boundary',linestyle='-')
                    
                    bound=self.get('c_core_mass')[modstart:modstop]-ylims[0]
                    bound1=(bound-ylims1[0])*float(yscale)
                    ax.plot(xxx,bound1,label='He boundary',linestyle='--')
                    
                    bound=self.get('o_core_mass')[modstart:modstop]-ylims[0]
                    bound1=(bound-ylims1[0])*float(yscale)
                    ax.plot(xxx,bound1,label='C boundary',linestyle='-.')
                    
                    bound=self.get('si_core_mass')[modstart:modstop]-ylims[0]
                    bound1=(bound-ylims1[0])*float(yscale)
                    ax.plot(xxx,bound1,label='C boundary',linestyle='-.')
                    
                    bound=self.get('fe_core_mass')[modstart:modstop]-ylims[0]
                    bound1=(bound-ylims1[0])*float(yscale)
                    ax.plot(xxx,bound1,label='C boundary',linestyle='-.')
                
                except:
                    pass

        ax.axis([xlims[0],xlims[1],ylims[0],ylims[1]])

        if plot_radius == True:
            ax2=pyl.twinx()
            ax2.plot(xxx,np.log10(self.get('he4_boundary_radius')[modstart:modstop]),label='He boundary radius',color='k',linewidth=1.,linestyle='-.')
            ax2.plot(xxx,self.get('log_R')[modstart:modstop],label='radius',color='k',linewidth=1.,linestyle='-.')
            ax2.set_ylabel('log(radius)')
        if rad_lines == True:
            ax2=pyl.twinx()
            for i in range(len(masses)):
                ax2.plot(modno,np.log10(rads[i]),color='k')

        if outfile[-3:]=='png':
            fig.savefig(outfile,dpi=300)
        elif outfile[-3:]=='eps':
            fig.savefig(outfile,format='eps')
        elif outfile[-3:]=='pdf':
            fig.savefig(outfile,format='pdf')
        if showfig == True:
            pyl.show()
# we may or may not need this below
#        fig.clear()

    def find_first_TP(self):

        '''
        Find first TP of the model.
        '''

        h1_boundary_mass  = self.get('h1_boundary_mass')
        he4_boundary_mass = self.get('he4_boundary_mass')
        star_mass         = self.get('star_mass')
        mx2_bot           = self.get('mx2_bot')*star_mass
        he_lumi           = self.get('log_LHe')
        h_lumi            = self.get('log_LH')
        lum_array=[]
        activate=False
        models=[]
        for i in range(len(h1_boundary_mass)):
            if (h1_boundary_mass[i]-he4_boundary_mass[i] <0.1) and (he4_boundary_mass[i]>0.2):
                if (mx2_bot[i]>he4_boundary_mass[i]) and (he_lumi[i]>h_lumi[i]):
                    activate=True
                    lum_array.append(he_lumi[i])
                    models.append(i)
                if (activate == True) and (he_lumi[i]<h_lumi[i]):
                    break
        t0_model = models[np.argmax(lum_array)]
        return t0_model

    def find_TPs_and_DUPs(self, percent=5., makefig=False):
        ''' 
        Function which finds TPs and uses the calc_DUP_parameter
        function.  To calculate DUP parameter evolution dependent of
        the star or core mass.
        
        Parameters
        ----------
        fig : integer
            Figure number to plot.
        t0_model : integer
            First he-shell lum peak.
        percent : float
            dredge-up is defined as when the mass dredged up is a certain
            percent of the total mass dredged up during that event, which
            is set by the user in this variable.
            The default is 5.
        makefig : 
            do you want a figure to be made?
            
        Returns
        -------
        TPmods : array
            model numbers at the peak of each thermal pulse
        DUPmods : array
            model numbers at the dredge-up, where dredge-up is defined as
            when the mass dredged up is a certain percent of the total mass
            dredged up during that event, which is set by the user
        TPend : array
            model numbers at the end of the PDCZ for each TP
        lambda : array
            DUP efficiency for each pulse
        '''

        t0_model=self.find_first_TP()
        t0_idx=(t0_model-self.get("model_number")[0])
        first_TP_he_lum=10**(self.get("log_LHe")[t0_idx])
        he_lum=10**(self.get("log_LHe")[t0_idx:])
        h_lum=10**(self.get("log_LH")[t0_idx:])
        model=self.get("model_number")[t0_idx:]
        h1_bndry=self.get("h1_boundary_mass")[t0_idx:]
        # SJ find TPs by finding local maxima in He-burning luminosity and
        # checking that the he_lum is greater than the h_lum:
        maxima=[0]
        for i in range(2,len(model)-1):
            if he_lum[i] > he_lum[i-1] and he_lum[i] > he_lum[i+1]:
                if he_lum[i-1] > he_lum[i-2] and he_lum[i+1] > he_lum[i+2]:
                    if he_lum[i] > h_lum[i]:
                        maxima.append(i)
    
        # find DUPs when h-boundary first decreases by more than XX% of the total DUP
        # depth:
        DUPs=[]
        TPend=[]
        maxDUPs=[]
        for i in range(len(maxima)):
            idx1=maxima[i]
            try:
                idx2=maxima[i+1]
            except IndexError:
                idx2=-1
            bound=h1_bndry[idx1:idx2]
            bound0=bound[0]
            if bound0==min(bound) or bound0 < min(bound): # then no DUP
                DUP=idx1
                DUPs.append(DUP)
                maxDUPs.append(DUP)
            else:
                maxDUPs.append(idx1+bound.argmin()) # model number of deepest extend of 3DUP
                maxDUP=bound0-min(bound) # total mass dredged up in DUP
                db=bound - bound[0]
                db_maxDUP = db / maxDUP
                DUP=np.where(db_maxDUP <= -float(percent)/100.)[0][0]
                DUPs.append(DUP+idx1)
#                # Alternative definition, where envelope reaches mass coordinate
#                # where top of PDCZ had resided during the TP:
#                top=self.get('mx2_top')[idx1]
#                DUP=np.abs(bound-top).argmin()
#                DUPs.append(DUP+idx1)
    
        # find end of PDCZ by seeking from TP peak and checking mx2_bot:
            mx2b=self.get('mx2_bot')[t0_idx:][idx1:idx2]
            for i in range(len(mx2b)):
                if mx2b[i]==0.:
                    endTP=i+idx1
                    TPend.append(endTP)
                    break
        
        # 3DUP efficiency:
        lambd=[0.]
        for i in range(1,len(maxima)):
            dmenv = h1_bndry[maxima[i]] - h1_bndry[maxDUPs[i-1]]
            dmdredge = h1_bndry[maxima[i]] - h1_bndry[maxDUPs[i]]
            lambd.append(dmdredge/dmenv)

        TPmods = maxima + t0_idx
        DUPmods = DUPs + t0_idx
        TPend = TPend + t0_idx
    
        return TPmods, DUPmods, TPend, lambd

    def find_TP_attributes(self, fig, t0_model, color, marker_type,
                           h_core_mass=False, no_fig=False):
        '''
        Function which finds TPs and uses the calc_DUP_parameter
        function.  To calculate DUP parameter evolution dependent of
        the star or core mass.
        
        Parameters
        ----------
        fig : integer
        Figure number to plot.
        t0_model : integer
        First he-shell lum peak
        color : string
        Color of the plot.
        marker_type : string
        marker type.
        h_core_mass : boolean, optional
        If True: plot dependence from h free core , else star mass.
        The default is False.
        no_fig : boolean, optional
        The default is False.
        
        '''
        
        #if len(t0_model)==0:
        
        t0_idx=(t0_model-self.get("model_number")[0])
        first_TP_he_lum=10**(self.get("log_LHe")[t0_idx])
        he_lum=10**(self.get("log_LHe")[t0_idx:])
        h_lum=10**(self.get("log_LH")[t0_idx:])
        model=self.get("model_number")[t0_idx:]
        h1_bndry=self.get("h1_boundary_mass")[t0_idx:]
        #define label
        z=self.header_attr["initial_z"]
        mass=self.header_attr["initial_mass"]
        leg=str(mass)+"M$_{\odot}$ Z= "+str(z)
        peak_lum_model=[]
        h1_mass_tp=[]
        h1_mass_min_DUP_model=[]
        ##TP identification with he lum if within 1% of first TP luminosity
        perc=0.01
        min_TP_distance=300 #model
        lum_1=[]
        model_1=[]
        h1_mass_model=[]
        TP_counter=0
        new_TP=True
        TP_interpulse=False
        interpulse_counter=0
        for i in range(len(he_lum)):
            if (h_lum[i]>he_lum[i]):
                TP_interpulse=True
                interpulse_counter+=1
            if (h_lum[i]<he_lum[i]):
                interpulse_counter=0
                new_TP=True
                TP_interpulse=False
                if i > 0:
                    h1_mass_1.append(h1_bndry[i])
                    h1_mass_model.append(model[i])
            #print i
            if i ==0:
                #peak_lum_model=[t0_model]
                #TP_counter=1 #for first TP
                lum_1.append(first_TP_he_lum)
                model_1.append(t0_model)
                h1_mass_1=[h1_bndry[0]]
                h1_mass_model=[t0_model]
            else:
                lum_1.append(he_lum[i])
                model_1.append(model[i])

            if (new_TP == True and TP_interpulse==True and interpulse_counter >200): #and (model[i] - t0_model      >min_TP_distance):
                #if (he_lum[i]> (perc*first_TP_he_lum)) or (i == len(he_lum)-1):
                #if (model[i] - model_1[-1]     >min_TP_distance):
                #calculate maximum of peak lum of certain TP
                max_value=np.array(lum_1).max()
                max_index = lum_1.index(max_value)
                #print max_index,i
                peak_lum_model.append(model_1[max_index])
                #for DUP calc
                max_lum_idx=h1_mass_model.index(model_1[max_index])
                min_value=np.array(h1_mass_1[max_lum_idx:]).min()
                min_index = h1_mass_1.index(min_value)
                h1_mass_min_DUP_model.append(h1_mass_model[min_index])
                TP_counter+=1
                lum_1=[]
                model_1=[]
                h1_mass_1=[]
                h1_mass_model=[]
                new_TP=False
        #TP_interpulse=False

        #print peak_lum_model
        #print h1_mass_min_DUP_model
        #print h1_mass_tp
        modeln=[]
        for i in range(len(peak_lum_model)):
            modeln.append(peak_lum_model[i])
            modeln.append(h1_mass_min_DUP_model[i])
        if no_fig==True:
            self.calc_DUP_parameter(fig,modeln,leg,color,marker_type,h_core_mass)
        return peak_lum_model,h1_mass_min_DUP_model

    def calc_DUP_parameter(self, fig, modeln, leg, color, marker_type,
                           h_core_mass=False):
        ''' 
        Method to calculate the DUP parameter evolution for different
        TPs specified specified by their model number.

        Parameters
        ----------
        fig : integer
            Figure number to plot.
        modeln : list
            Array containing pairs of models each corresponding to a
            TP. First model where h boundary mass will be taken before
            DUP, second model where DUP reaches lowest mass.
        leg : string
            Plot label.
        color : string
            Color of the plot.
        marker_type : string
            marker type.
        h_core_mass : boolean, optional
            If True: plot dependence from h free core , else star mass.
            The default is False.
            
        '''
        number_DUP=(len(modeln)/2 -1) #START WITH SECOND
        h1_bnd_m=self.get('h1_boundary_mass')
        star_mass=self.get('star_mass')
        age=self.get("star_age")
        firstTP=h1_bnd_m[modeln[0]]
        first_m_dredge=h1_bnd_m[modeln[1]]
        DUP_parameter=np.zeros(number_DUP)
        DUP_xaxis=np.zeros(number_DUP)
        j=0
        for i in np.arange(2,len(modeln),2):
            TP=h1_bnd_m[modeln[i]]
            m_dredge=h1_bnd_m[modeln[i+1]]
            if i ==2:
                last_m_dredge=first_m_dredge
            #print "testest"
            #print modeln[i]
            if h_core_mass==True:
                DUP_xaxis[j]=h1_bnd_m[modeln[i]]                        #age[modeln[i]] - age[modeln[0]]
            else:
                DUP_xaxis[j]=star_mass[modeln[i]]
            #DUP_xaxis[j]=modeln[i]
            DUP_parameter[j]=(TP-m_dredge)/(TP-last_m_dredge)
            last_m_dredge=m_dredge
            j+=1

        pl.figure(fig)
        pl.rcParams.update({'font.size': 18})
        pl.rc('xtick', labelsize=18)
        pl.rc('ytick', labelsize=18)

        pl.plot(DUP_xaxis,DUP_parameter,marker=marker_type,markersize=12,mfc=color,color='k',linestyle='-',label=leg)
        if h_core_mass==True:
            pl.xlabel("$M_H$",fontsize=20)
        else:
            pl.xlabel("M/M$_{\odot}$",fontsize=24)
        pl.ylabel("$\lambda_{DUP}$",fontsize=24)
        pl.minorticks_on()
        pl.legend()



class star_log(history_data):
    ''' 
    Class derived from history_data class (copy). Existing just (for
    compatibility reasons) for older mesa python scripts.
    
    '''



# below are some utilities that the user typically never calls directly


def _read_mesafile(filename,data_rows=0,only='all'):
    ''' private routine that is not directly called by the user'''
    f=open(filename,'r')
    vv=[]
    v=[]
    lines = []
    line  = ''
    for i in range(0,6):
        line = f.readline()
        lines.extend([line])

    hval  = lines[2].split()
    hlist = lines[1].split()
    header_attr = {}
    for a,b in zip(hlist,hval):
        header_attr[a] = float(b)
    if only is 'header_attr':
        return header_attr

    cols    = {}
    colnum  = lines[4].split()
    colname = lines[5].split()
    for a,b in zip(colname,colnum):
        cols[a] = int(b)

    data = []
    for i in range(data_rows):
        # writing reading status
        percent = int(i*100/data_rows)
        sys.stdout.flush()
        sys.stdout.write("\r reading " + "...%d%%" % percent)
        line = f.readline()
        v=line.split()
        try:
            vv=np.array(v,dtype='float64')
        except ValueError:
            for item in v:
                if item.__contains__('.') and not item.__contains__('E'):
                    v[v.index(item)]='0'
        data.append(vv)
    print '\n'
    f.close()
    a=np.array(data)
    data = []
    return header_attr, cols, a


def _cleanstarlog(file_in):
    ''' 
    cleaning history.data or star.log file, e.g. to take care of
    repetitive restarts.

    private, should not be called by user directly

    Parameters
    ----------
    file_in : string
        Typically the filename of the mesa output history.data or
        star.log file, creates a clean file called history.datasa or
        star.logsa.

    (thanks to Raphael for providing this tool)

    '''

    file_out=file_in+'sa'
    f = open(file_in)
    lignes = f.readlines()
    f.close()

    nb    = np.array([],dtype=int)   # model number
    nb    = np.concatenate((nb    ,[  int(lignes[len(lignes)-1].split()[ 0])]))
    nbremove = np.array([],dtype=int)   # model number
    i=-1

    for i in np.arange(len(lignes)-1,0,-1):
        line = lignes[i-1]

        if i > 6 and line != "" :
            if int(line.split()[ 0])>=nb[-1]:
                nbremove = np.concatenate((nbremove,[i-1]))
            else:
                nb = np.concatenate((nb    ,[  int(line.split()[ 0])]))
    i=-1
    for j in nbremove:
        lignes.remove(lignes[j])

    fout=file(file_out,'w')
    for j in np.arange(len(lignes)):
        fout.write(lignes[j])
    fout.close()




#
# NuGridpy - Tools for accessing and visualising NuGrid data.
#
# Copyright 2007 - 2014 by the NuGrid Team.
# All rights reserved. See LICENSE.
#

""" 
Analyze and visualize the output of single-zone ppn simulations

Contains:

class xtime: for analyzing x-time.data time evolutions

class abu_vector: for analyzing iso_massfxxxx.DAT time evolutions

PPN Assumptions
===============
The first non white space character in a header line is a '#'.  Header
attributes are separated from their value by white space or by white
space surrounding an equals sign.

A Header attribute is separated by the previous Header attribute by
white space or a line break.

There are only 6 data columns.  The first being the number, second
being Z, third being A, fourth being isomere state, fifth being
abundance_yps, and sixth the element name.

The first five columns consist purely of numbers, no strings are
allowed. Of the values in the final column, the name column, the first
two are letters specifying the element name, and the rest are spaces or
numbers (in that strict order), except for the element names: Neut and
Prot.

All the profile files in the directory have the same cycle attributes.
The cycle numbers of the 'filename'+xxxxx start at 0.

PPN files always end in .DAT and are not allowed any '.'.

There can not be any blank lines in the data files.

No cycle numbers are skipped, ie if cycle 0 and 3 are present, 1 and 2
must be here as well.

"""

from numpy import *
from data_plot import *
import matplotlib
from matplotlib.pylab import *
from data_plot import *
from utils import *
import os
import utils

import pdb

class xtime(DataPlot):
    ''' 
    read and plot x-time.dat output files.

    Parameters
    ----------
    sldir : string, optional
        The directory of the specified filename.  The default is './'.
    fname : string, optional
        Specify alternative filename of file of type x-time.dat.  The
        default is 'x-time.dat'.
        
    Returns
    -------
    xtime
        A xtime instance.

    Examples
    --------
    >>> import ppn
    >>> f=ppn.xtime()
    There are 1099 species found.
    There are 19 time steps found.

    >>> f.col
    f.col_num  f.col_tot  f.cols

    >>> f.cols[0:10]
    ['age',
    't9',
    'rho',
    'sum_yps',
    'NEUT',
    'PROT',
    'H   2',
    'HE  3',
    'HE  4',
    'BE  7']

    >>> f.plot('HE  4')
    age HE  4

    >>> f.plot('C  12')
    age C  12

    >>> f.plot('PROT')

    '''
    sldir  = '' #Standard Directory
    data = []
    cols=[]     #list of column attribute names
    col_num = []#Dict of column attribute names and their associated values
    xdat = []
    ydat = []

    def __init__(self, sldir='./', fname='x-time.dat'):
        ''' 
        A xtime instance

        Parameters
        ----------
        sldir : string, optional
            The directory of the specified filename.  The default is './'.
        fname : string, optional
            Specify alternative filename of file of type x-time.dat.  The
            default is 'x-time.dat'.

        Returns
        -------
        xtime
            A xtime instance.

        '''
        self.sldir= sldir

        if not os.path.exists(sldir):  # If the path does not exist
            print 'error: Directory, '+sldir+ ' not found'
            print 'Now returning None'
            return None
        else:
            self.data, self.col_num, self.cols, self.col_tot, self.ilines =\
            self._readFile(fname,sldir)


    def _readFile(self, fname, sldir):
        ''' 
        Private method that reads in the data file and organizes it
        within this object.

        '''
        if sldir.endswith('/'):
            fname = str(sldir)+str(fname)
        else:
            fname = str(sldir)+'/'+str(fname)
        f=open(fname,'r')

        # read header line
        line=f.readline()
        cols  = []
        ispec = 0
        for i in range(1,len(line.split('|'))):
            col = line.split('|')[i].strip()
            if '-' in col:
                ispec += 1
                col   = col.split('-')[1]
            cols.append(col)
            col_num={}
        col_tot = len(cols)

        print 'number of species: ', str(ispec)
        print 'number of cols: ', str(col_tot)


        col_num={}
        for a,b in zip(cols,range(col_tot)):
            col_num[a]=b

        # read remainder of the file
        lines=f.readlines()
        data=[]
        for i in range(len(lines)):
            v=lines[i].split()
            vv=array(v,dtype='float')
            data.append(vv)
        ilines=i
        print "There are "+str(ilines)+" time steps found."
        return data,col_num,cols,col_tot,ilines

    def get(self, col_str):
        ''' 
        get one data column with the data

        Parameters
        ----------
        col_str : string
            One of the column strings in self.cols.
            
        '''
        data_column=zeros(self.ilines)
        for i in range(self.ilines):
            data_column[i]=self.data[i][self.col_num[col_str]]
        return data_column

    def plot_xtime(self, y, x='t_y', label='default', labelx=None,
                   labely=None ,title=None, shape='.', logx=False,
                   logy=True, base=10):
        ''' 
        make a simple plot of two columns against each other.
        
        An example would be instance.plot_xtime('PB206', label='PB206 vs t_y'
        Recomend using the plot function DataPlot.plot() it has more
        functionality.
        
        Parameters
        ----------
        Y : string
            Column on Y-axis.
        X : string, optional
            Column on X-axis.  The default is "t_y".
        label : string, optional
            Legend label.  The default is "default".
        labelX : string, optional
            The label on the X axis.  The default is None.
        labelY : string, optional
            The label on the Y axis.  The default is None.
        title : string, optional
            The Title of the Graph.  The default is None.
        shape : string, optional
            What shape and colour the user would like their plot in.
            The default is '.'.
        logX : boolean, optional
            A boolean of weather the user wants the x axis
            logarithmically.  The default is False.
        logY : boolean, optional
            A boolean of weather the user wants the Y axis
            logarithmically.  The default is True.
        base : integer, optional
            The base of the logarithm.  The default is 10.
        
        Notes
        -----
        For all possable choices visit,
        <http://matplotlib.sourceforge.net/api/pyplot_api.html#matplotlib.pyplot.plot>
        
        '''
        if label is 'default':
            lab_str=y
        else:
            lab_str=label

        try:
            self.get(x)
        except KeyError:
            x='age'

        DataPlot.plot(self,x,y,legend=lab_str,labelx=labelx, labely=labely,
                      title=title, shape=shape,logx=logx, logy=logy, base=base)
        ''' 
        print X,Y
        xdat=self.get(X)
        ydat=self.get(Y)
        self.xdat = xdat
        self.ydat = ydat


        plot(xdat,log10(ydat),label=lab_str)
        legend()
        '''

class abu_vector(DataPlot, Utils):
    ''' 
    Class for reading selem00xxxx.DAT files

    Parameters
    ----------
    sldir : string, optional
        where fname exists
    filenames : string, optional
        The default file names of the abundance vectors.  The
        defaults is "iso_massf".
    USEEPP : string, optional
        The default is "auto".

    Returns
    -------
    abu_vector
        A PPN instance.

    Examples
    --------
    >>> import ppn
    >>> p=ppn.abu_vector('./run')
    39 cycle numbers found in ./run
    Ranging from 0 to 38

    To find the cycle attributes:
    >>> p.cattrs
    ['mod', 'dzeit', 'agej', 't9', 'rho', 'densn', 'densp', 'densa']

    To find the data column attributes

    >>> p.dcols
    ['NUM', 'Z', 'A', 'ISOM', 'ABUNDANCE_MF', 'ISOTP']
    >>> p.get('Z',0)
    array([1, 2, 2, 4, 5, 3, 6, 6, 7, 7, 6, 7, 8, 8, 8, 9, 9, 9])
    >>> p.get('ABUNDANCE_MF',0)
    array([  1.43722000e-10,   1.00000000e-99,   9.81499000e-01,
    4.08738000e-20,   1.00000000e-99,   2.06944000e-21,
    3.42800000e-04,   9.62307000e-05,   1.05081000e-12,
    1.25518000e-02,   1.90131000e-08,   1.42230000e-07,
    4.98449000e-05,   4.80246000e-07,   4.24345000e-12,
    9.85201000e-17,   6.30866000e-16,   9.12726000e-11])

    or if the user wants the data from the first 3 cycles:

    >>> p.get('ABUNDANCE_MF',[0,1,2])
    [array([  1.43722000e-10,   ...,   9.81499000e-01,],
    array([  1.43722000e-10,   ...,   9.81499000e-01,],
    array([  1.43722000e-10,   ...,   9.81499000e-01,]
    >>> p.getElement('C 14',0)
    array([  1.50000000e+01,   6.00000000e+00,   1.40000000e+01,
    1.90131000e-08])
    >>> p.plot('abundance_yps', 'Z',0)

    plots data

    >>> p.iso_abund(0)

    Plots an isotope abundance distribution

    >>> p.abu_chart(0)

    Plots an isotope abundance chart

    One note about the plot functions, if instead of a single cycle the
    user inputs a list of cycles, the method will then, instead of
    plotting them, will then save a .png for each cycle.  Also if you
    just want a singular plot saved, the user can input their cycle,
    in a list like [0]. And that will save their plot.
    
    '''
    sldir = ''  #Standard Directory
    inputdir = '' # A copy of Standard Directory which never changes
    cattrs={} # cycle attributes
    dcols=[]  # list of the column attributes
    index=0   # index of were column data begins in the file
    files=[]  # list of files
    isotopes=[]# list of isotopes
    def __init__(self, sldir='./', filenames='iso_massf',
                 USEEPP='auto'):
        ''' 
        initial method of this class
        
        Parameters
        ----------
        sldir : string, optional
            where fname exists
        filenames : string, optional
            The default file names of the abundance vectors.  The
            defaults is "iso_massf".
        USEEPP : string, optional
            The default is "auto".

        Returns
        -------
        abu_vector
            A PPN instance.

        '''
        self.debug=False
        self._stable_names() # provides in addition to stable_el from
                             # utils also just the stable element names
        self.sldir = sldir
        self.inputdir = ''
        self.startdir = os.getcwd()
        self.cattrs=[]
        self.dcols=[]
        self.files=[]
        self.isotopes=[]
        if not os.path.exists(sldir):  # If the path does not exist
            print 'error: Directory, '+sldir+ ' not found'
            print 'Now returning None'
            return None
        f=os.listdir(sldir) # reads the directory

        def iniabmode(): #Where we go if dealing with iniab files in a USEEPP directory
            print "---> In iniab mode"
            for file in f:
            # Removes any files that are not ppn files
                if filenames in file and 'ppn' in file and '~' not in file \
                and '#' not in file and  file[-1] is 'n' \
                and file[-2] is 'p' and file[-3] is 'p'\
                and 'restart' not in file:
                    self.files.append(file)
            self.files.sort()

            if len(self.files)==0:
                # If there are no Files in the Directory
                print 'Error: no '+filenames+ ' named files exist in Directory'
                print 'Now returning None'
                return None
            fname=self.files[len(self.files)-1]
            self.dcols,self.index=self._readPPN(fname,sldir)
            indexp_cyc2filels={}  # created index pointer from mod (cycle
            i = 0                 # name) to index in files array
            for file in self.files:
                mod=utils.iniabu(file)
                indexp_cyc2filels[mod] = i
                i += 1
            self.indexp_cyc2filels = indexp_cyc2filels

            for i in xrange(len(self.files)):
                self.files[i]=self.sldir+self.files[i]
            print str(len(self.files))+' cycle numbers found in '+sldir
            print 'Ranging from 0 to '+str(len(self.files)-1)
            self.isotopes=self.get('ISOTP',self.files[0],numtype='file')

        def iso_massfmode():
            for file in f:
            # Removes any files that are not ppn files
                filelength=len(filenames)+4
                if filenames in file and 'DAT' in file and '~' not in file \
                and '#' not in file and len(file)>filelength \
                and 'restart' not in file:
                    self.files.append(file)
            self.files.sort()

            if len(self.files)==0:
                # If there are no Files in the Directory
                print 'Error: no '+filenames+ ' named files exist in Directory'
                print 'Now returning None'
                return None
            fname=self.files[len(self.files)-1]
            self.cattrs,self.dcols,self.index=self._readFile(fname,sldir)
            indexp_cyc2filels={}  # created index pointer from mod (cycle
            i = 0                 # name) to index in files array
            for file in self.files:
                mod=self.get('mod',fname=file,numtype='file')
                indexp_cyc2filels[mod] = i
                i += 1
            self.indexp_cyc2filels = indexp_cyc2filels

            for i in xrange(len(self.files)):
                self.files[i]=self.sldir+self.files[i]
            print str(len(self.files))+' cycle numbers found in '+sldir
            print 'Ranging from 0 to '+str(len(self.files)-1)
            self.isotopes=self.get('ISOTP',self.files[0],numtype='file')


        ''' 
        In this chunk of code we find whether the specified directory includes USEEPP
        and if it does, go to 'iniabmode' otherwise (or if USEEPP='off' is thrown) we
        deal with 'iso_massfmode'
        '''
        if ('USEEPP' in sldir or 'USEEPP' in self.startdir) and (USEEPP == 'on' or USEEPP == 'auto'):
            print '--> Going to iniab mode'
            filenames='iniab'
            iniabmode()
        elif ('USEEPP' in sldir or 'USEEPP' in self.startdir) and USEEPP == 'off':
            print "--> Skipping iniab mode (USEEPP == 'off')"
            iso_massfmode()
        elif ('USEEPP' not in sldir or 'USEEPP' not in self.startdir) and USEEPP == 'on':
            print "--> Error: Not in, or referencing, a USEEPP directory. Cannot enter iniab mode"
            print "--> Skipping iniab mode"
            iso_massfmode()
        else:
            iso_massfmode()

    def getCycleData(self, attri, fname, numtype='cycNum'):
        """ 
        In this method a column of data for the associated cycle
        attribute is returned.
        
        Parameters
        ----------
        attri : string
            The name of the attribute we are looking for.
        fname : string
            The name of the file we are getting the data from or the
            cycle number found in the filename.
        numtype : string, optional
            Determines whether fname is the name of a file or, the
            cycle number.  If it is 'file' it will then  interpret it as
            a file, if it is 'cycNum' it will then  interpret it as a
            cycle number.  The default is "cycNum".
            
        """

        fname=self.findFile(fname,numtype)

        if self.inputdir == '':
            self.inputdir = self.sldir      # This chunk of code changes into the directory where fname is,
        os.chdir(self.inputdir)                           # and appends a '/' to the directory title so it accesses the
        self.sldir=os.getcwd() + '/'              # file correctly

        f=open(fname,'r')
        lines=f.readlines()

        if self.inputdir != './':                               #This chunk of code changes back into the directory you started in.
            os.chdir(self.startdir)
            self.sldir = self.inputdir


        for i in range(len(lines)):
            lines[i]=lines[i].strip()

        for i in range(len(lines)):
            if lines[i].startswith('#'):
                lines[i]=lines[i].strip('#')
                tmp=lines[i].split()
                tmp1=[]
                for j in range(len(tmp)):
                    if tmp[j] != '=' or '':
                        tmp1.append(tmp[j])
                tmp=tmp1
                for j in range(len(tmp)):
                    if tmp[j]== attri:
                        try:
                            if '.' in tmp[j+1]:
                                return float(tmp[j+1])
                            else:
                                return int(tmp[j+1])
                        except ValueError:
                            return str(tmp[j+1])

            elif lines[i].startswith('H'):
                continue
            else:
                print 'This cycle attribute does not exist'
                print 'Returning None'
                return None




    def getColData(self, attri, fname, numtype='cycNum'):
        """ 
        In this method a column of data for the associated column
        attribute is returned.

        Parameters
        ----------
        attri : string
            The name of the attribute we are looking for.
        fname : string
            The name of the file we are getting the data from or the
            cycle number found in the filename.
        numtype : string, optional
            Determines whether fname is the name of a file or, the
            cycle number.  If it is 'file' it will then  interpret it as
            a file, if it is 'cycNum' it will then  interpret it as a
            cycle number.  The default is "cycNum".

        """
        fname=self.findFile(fname,numtype)
        f=open(fname,'r')
        for i in range(self.index+1):
            f.readline()
        lines=f.readlines()
        for i in range(len(lines)):
            lines[i]=lines[i].strip()
            lines[i]=lines[i].split()
        index=0
        data=[]

        while index < len (self.dcols):
            if attri== self.dcols[index]:
                break
            index+=1

        for i in range(len(lines)):

            if index==5 and len(lines[i])==7:
                data.append(str(lines[i][index].capitalize())+'-'\
                +str(lines[i][index+1]))
            elif index==5 and len(lines[i])!=7:
                tmp=str(lines[i][index])
                if tmp[len(tmp)-1].isdigit():
                    tmp1=tmp[0]+tmp[1]
                    tmp1=tmp1.capitalize()
                    tmp2=''
                    for j in range(len(tmp)):
                        if j == 0 or j == 1:
                            continue
                        tmp2+=tmp[j]
                    data.append(tmp1+'-'+tmp2)
                elif tmp=='PROT':
                    data.append('H-1')
                elif tmp==('NEUT'or'NEUTR'or'nn'or'N   1'or'N-1'):
                    data.append('N-1')
                else:
                    data.append(tmp)
            elif index==0:
                data.append(int(lines[i][index]))
            else:
                data.append(float(lines[i][index]))

        return array(data)

    def getElement(self, attri, fname, numtype='cycNum'):
        ''' 
        In this method instead of getting a particular column of data,
        the program gets a particular row of data for a particular
        element name.
        
        attri : string
            The name of the attribute we are looking for. A complete
            list of them can be obtained by calling

            >>> get('element_name')
        fname : string
            The name of the file we are getting the data from or the
            cycle number found in the filename.
        numtype : string, optional
            Determines whether fname is the name of a file or, the
            cycle number.  If it is 'file' it will then  interpret it as
            a file, if it is 'cycNum' it will then  interpret it as a
            cycle number.  The default is "cycNum".
       
        Returns
        -------
        array
            A numpy array of the four element attributes, number, Z, A
            and abundance, in that order.
            
        Notes
        -----
        Warning
        '''
        element=[] #Variable for holding the list of element names
        number=[]  #Variable for holding the array of numbers
        z=[]       #Variable for holding the array of z
        a=[]       #Variable for holding the array of a
        abd=[]     #Variable for holding the array of Abundance
        data=[]    #variable for the final list of data

        fname=self.findFile(fname,numtype)
        f=open(fname,'r')
        for i in range(self.index+1):
            f.readline()
        lines=f.readlines()
        for i in range(len(lines)):
            lines[i]=lines[i].strip()
            lines[i]=lines[i].split()
        index=0
        data=[]

        while index < len (self.dcols):
            if attri== self.dcols[index]:
                break
            index+=1

        element=self.get(self.dcols[5],fname,numtype)
        number=[]
        z=[]
        a=[]
        isom=[]
        abd=[]
        for i in range(len(lines)):
            number.append(int(lines[i][0]))
            z.append(float(lines[i][1]))
            isom.append(float(lines[i][2]))
            abd.append(float(lines[i][1]))
        index=0 #Variable for determing the index in the data columns


        while index < len(element):
            if attri == element[index]:
                break

            index+=1

        data.append(number[index])
        data.append(z[index])
        data.append(a[index])
        data.append(isom[index])
        data.append(abd[index])

        return array(data)



    def get(self, attri, fname=None, numtype='cycNum', decayed=False):
        ''' 
        In this method all data for an entire cycle (basically the
        content of an iso_massfnnnn.DAT file) or a column of data for
        the associated attribute is returned.

        Parameters
        ----------
        attri : string or integer
            If attri is a string, attri is the cycle or name of the
            attribute we are looking for.
            
            If attri is an integer, attri is the cycle number (cycle arrays
            are not supported).
        fname : string, optional
            If attri is a string, fname is the name of the file we are
            getting the data from or the cycle number found in the
            filename, or a List of either cycles or filenames.  If fname
            is None, the data from all cycles is returned.
            
            If attri is an integer, then fname is not supported.
            
            The default is None. 
        numtype : string, optional
            If attri is a string, numtype determines whether fname is
            the name of a file or, the cycle number.  If numtype is
            'file' it will then  interpret fname as a file.  If numtype
            is 'cycNum' it will then interpret fname as a cycle number.
            
            If attri is an Integer, then numtype is not supported.
            
            The default is "cycNum".
        decayed : boolean, optional
            If attri is a string, then decayed is not supported.
            
            If attri is an integer, then get instantaneously decay
            abundance distribution.
            
            The default is False.
            
        Returns
        -------
        array
            If attri is a string, data in the form of a numpy array is
            returned.
            
            If attri is an integer, Nothing is returned.
            
        Notes
        -----
        If attri is an integer, then the following variables will be
        added to the instance.
        
        a_iso_to_plot: mass number of plotted range of species.
        
        isotope_to_plot: corresponding list of isotopes.
        
        z_iso_to_plot: corresponding charge numbers.
        
        el_iso_to_plot: corresponding element names.
        
        abunds: corresponding abundances.

        isom: list of isomers with their abundances.

        '''
        if isinstance(attri,int):
            print "Calling get method in cycle mode, adding a_iso_to_plot, z.. el.. isotope.. isotope... to instance"
            self._getcycle(attri,decayed)
        elif isinstance(attri,str):
            data=self._getattr(attri,fname,numtype)
            return data

    def _getcycle(self, cycle, decayed=False):
        ''' Private method for getting a cycle, called from get.'''
        yps=self.get('ABUNDANCE_MF', cycle)
        z=self.get('Z', cycle) #charge
        a=self.get('A', cycle) #mass
        isomers=self.get('ISOM', cycle)

        a_iso_to_plot,z_iso_to_plot,abunds,isotope_to_plot,el_iso_to_plot,isom=\
            self._process_abundance_vector(a,z,isomers,yps)
        self.a_iso_to_plot=a_iso_to_plot
        self.isotope_to_plot=isotope_to_plot
        self.z_iso_to_plot=z_iso_to_plot
        self.el_iso_to_plot=el_iso_to_plot
        self.abunds=array(abunds)
        self.isom=isom

        if decayed:
            try:
                self.decay_idp
            except AttributeError:
                print "WARNING: decayed in _getcycle ignores isomers " \
                    "and will decay alpha-unstable p-rich nuclei as if they were beta+ stable."
                print "Initialising decay index pointers ...."
                self.decay_indexpointer() # provides self.decay_idp and
            ind_tmp=self.idp_to_stables_in_isostoplot

            isotope_decay=array(isotope_to_plot)[ind_tmp]
            z_iso_decay=array(z_iso_to_plot)[ind_tmp]
            a_iso_decay=array(a_iso_to_plot)[ind_tmp]
            el_iso_decay=array(el_iso_to_plot)[ind_tmp]
            abunds_decay=zeros(len(ind_tmp), dtype='float64')
            for i in xrange(len(isotope_to_plot)):
                idp=where(isotope_decay==isotope_to_plot[self.decay_idp[i]])[0] # points from
                # i on isotope_to_plot scale to decay target_on_decayed array scale
                abunds_decay[idp] += abunds[i]

            if self.debug:
                print "Decayed array:"
                for i in xrange(len(ind_tmp)):
                    print isotope_decay[i], z_iso_decay[i], a_iso_decay[i], el_iso_decay[i], abunds_decay[i]

            self.a_iso_to_plot=a_iso_decay
            self.isotope_to_plot=isotope_decay
            self.z_iso_to_plot=z_iso_decay
            self.el_iso_to_plot=el_iso_decay
            self.abunds=abunds_decay


    def _getattr(self, attri, fname=None, numtype='cycNum'):
        ''' Private method for getting an attribute, called from get.'''
        if str(fname.__class__)=="<type 'list'>":
            isList=True
        else:
            isList=False

        data=[]
        if fname==None:
            fname=self.files
            numtype='file'
            isList=True
        if isList:
            for i in xrange(len(fname)):
                if attri in self.cattrs:
                    data.append(self.getCycleData(attri,fname[i],numtype))
                elif attri in self.dcols:
                    data.append(self.getColData(attri,fname[i],numtype))
                elif attri in self.get('ISOTP',fname,numtype):
                    data.append(self.getElement(attri,fname[i],numtype))
                else:
                    print 'Attribute '+attri+ ' does not exist'
                    print 'Returning none'
                    return None

        else:
            if attri in self.cattrs:
                return self.getCycleData(attri,fname,numtype)
            elif attri in self.dcols:
                return self.getColData(attri,fname,numtype)
            elif attri in self.get('ISOTP',fname,numtype):
                return self.getElement(attri,fname,numtype)
            else:
                print 'Attribute '+attri+ ' does not exist'
                print 'Returning none'
                return None

        return data


    def _readPPN(self, fname, sldir):
        ''' 
        Private method that reads in and organizes the .ppn file
        Loads the data of the .ppn file into the variable cols.

        '''
        if sldir.endswith(os.sep):
                    #Making sure fname will be formatted correctly
            fname = str(sldir)+str(fname)
        else:
            fname = str(sldir)+os.sep+str(fname)
            self.sldir+=os.sep
        f=open(fname,'r')
        lines=f.readlines()
        for i in range(len(lines)):
            lines[i]=lines[i].strip()

        cols = ['ISOTP', 'ABUNDANCE_MF'] #These are constant, .ppn files have no header to read from
        for i in range(len(lines)):
            if not lines[i].startswith('H'):
                index = i-1
                break

        return cols, index


    def _readFile(self, fname, sldir):
        ''' 
        private method that reads in and organizes the .DAT file
        Loads the data of the .DAT File into the variables cattrs and cols.
        In both these cases they are dictionaries, but in the case of cols,
        it is a dictionary of numpy array exect for the element ,
        element_name where it is just a list

        '''
        cattrs=[]
        if sldir.endswith(os.sep):
            #Making sure fname will be formatted correctly
            fname = str(sldir)+str(fname)
        else:
            fname = str(sldir)+os.sep+str(fname)
            self.sldir+=os.sep
        f=open(fname,'r')
        lines=f.readlines()
        for i in range(len(lines)):
            lines[i]=lines[i].strip()


        cols=lines[0].strip('H')
        cols=cols.strip()
        cols=cols.split()
        for i in range(len(lines)):
            if lines[i].startswith('#'):
                # if it is a cycle attribute line
                lines[i]=lines[i].strip('#')
                tmp=lines[i].split()
                tmp1=[]
                for j in range(len(tmp)):
                    if tmp[j] != '=' or '':
                        tmp1.append(tmp[j])
                tmp=tmp1

                j=0
                while j <len(tmp):
                    cattrs.append(tmp[j])
                    j+=2


            elif not lines[i].startswith('H'):
                index = i-1
                break


        return cattrs,cols, index

    def findFile(self, fname, numtype):
        """ 
        Function that finds the associated file for fname when Fname is
        time or NDump.
        
        Parameters
        ----------
        fname : string
            The name of the file we are looking for.
        numType : string
            Designates how this function acts and how it interprets
            fname.  If numType is 'file', this function will get the
            desired attribute from that file.  If numType is 'cycNum',
            this function will get the desired attribute from that file
            with fname's model number.

        """
        numType=numtype.upper()
        if numType == 'FILE':
                #do nothing
            return fname
        elif numType == 'CYCNUM':
            try:
                fname = int(fname)
            except ValueError:
                print 'Improper choice:'+ str(fname)
                print 'Reselecting as 0'
                fname = 0
                print 'Using '+self.files[fname]
        try:
            return self.files[self.indexp_cyc2filels[fname]]
        except IndexError:
            mods = array(self.get('mod'), dtype=int)
            if fname not in mods:
                print 'You seem to try to plot a cycle that is not present: '+str(fname)
                fname = mods[-1]
                print 'I will assume you want to plot the last cycle in the run: '+str(fname)
                print '[I am not 100% sure this escape is debugged. You better do this again with'
                print 'the correct input.]'
                return self.files[fname]




#
# NuGridpy - Tools for accessing and visualising NuGrid data.
#
# Copyright 2007 - 2014 by the NuGrid Team.
# All rights reserved. See LICENSE.
#

""" 
Ascii_table.py: read and write simple ascii tables

By Daniel Alexander Bertolino Conti
Fall 2010
If the user find any bugs or errors, please email fherwig@uvic.ca.

Assumptions for ascii Files:

    Headers are always at the beginning of the file and start with a
    capital H (default, can be reset).  The next line after the header
    lines, is a line of column attribute names.  The column attribute
    names are separated by ' ' by default or whatever the user
    dictates to the class.  Data columns are seperated by spaces and
    each data attribute contains no spaces.  All the data columns are
    of equal length.  Any file name that has 'trajectory' or
    'Trajectory' in it, is assumed to be a trajectory type file.

Assumptions for Trajectory Files:

    The first three lines start with a '#'.  The first of these contains
    'time', 'T' and 'rho', each seperated by a space.  The second is
    '# YRS/SEC; T8K/T9K; CGS/LOG' which are the choices for the header
    attributes AGEUNIT, TUNIT, and RHOUNIT can be.  The third is
    something like 'FORMAT:' The four lines afther those three are the
    header attributes.  Each line has one header attribute, followd by
    a = and then followed after by the value.  After these seven lines
    comes three columns of data, the first is is associated with
    'time', the second with 'T' and the third with rho.

"""
from numpy import *
from data_plot import *
import matplotlib.pylab as pyl
import matplotlib.pyplot as pl
import os

class ascii_table(DataPlot):
    ''' 
    Data structure to read simple data tables and trajectory data
    tables.

    Init method that reads in ascii type files and trajectory type
    files.  By default this method reads ascii type files.  If the user
    wants a trajectory file read, either the file must have 'trajectory'
    in the filename or the user must set the datatype='trajectory'.

    Parameters
    ----------
    filename : string
        The name of the file we are looking at, or writeing to.
    sldir : string, optional
        Standard directory of filename. The default is '.'.
    sep : string, optional
        The seperator that seperates column attributes in filename.  The
        defaults is '  '.
    datatype : string, optional
        What type of ascii table you are reading and or writeing.  The
        only two options currently are 'normal' and 'trajectory'.  The
        default is 'normal'.
    Headers : list, optional
        A list of Header strings or if the file being writen is of type
        trajectory, this is a dictionary of header attributes and their
        associated values. The default is [].
    dcols : list, optional
        A list of data attributes. The default is [].
    data : list, optional
        A list of lists (or of numpy arrays) of columns of data.  The
        default is [].
    header_char : caracter, optional
        The character that indicates a header lines. The default is 'H'.
    read : boolean, optional
        Boolean of weather this is reading or writing a file.  The
        default is True.
    headerlines : list, optional
        Additional list of strings of header data, only used in
        trajectory data Types. The default is [].

    Examples
    --------
    >>> from ascii_table import *
    >>> p=ascii_table('c12cg.dat')
    >>> p.hattrs
    ['1 12  6  1  1  1  0  0  0  1 13  7  0', '55   1.943', 'c12pg']
    >>> a.dcols
    ['upper', 'lower', 'T9', 'ado(gs)', 'ado/CA88', 'tt/gs', 'ado/fit', 'CA88', 'fitted']
    >>> a.get('upper')
    [1.3400000000000001e-24, ... 1590.0]
    >>> a.plot('T9', 'ado/CA88') #plots data

    Also there is a global method called write that will allow a user
    to write ascii Files and trajectory files.

    One Can call this method like

    >>> write('file.txt',dcols,data,headers)

    Where Dcols is a list of data attributes, data is a list of lists
    of data and headers is a list of strings that are each a header
    attribute.
    
    See Also
    --------
    write()

    '''

    def __init__(self, filename, sldir='.', sep='  ', datatype='normal',
                 headers=[], dcols=[], data=[], header_char='H',
                 read=True, headerlines=[]):
        ''' 
        Init method that reads in ascii type files and trajectory type
        files.  By default this method reads ascii type files.  If the
        user wants a trajectory file read, either the file must have
        'trajectory' in the filename or the user must set the
        datatype='trajectory'.

        Parameters
        ----------
        filename : string
            The name of the file we are looking at, or writeing to.
        sldir : string, optional
            Standard directory of filename. The default is '.'.
        sep : string, optional
            The seperator that seperates column attributes in filename.
            The defaults is '  '.
        datatype : string, optional
            What type of ascii table you are reading and or writeing.
            The only two options currently are 'normal' and
            'trajectory'.  The default is 'normal'.
        Headers : list, optional
            A list of Header strings or if the file being writen is of
            type trajectory, this is a dictionary of header attributes
            and their associated values. The default is [].
        dcols : list, optional
            A list of data attributes. The default is [].
        data : list, optional
            A list of lists (or of numpy arrays) of columns of data.
            The default is [].
        header_char : caracter, optional
            The character that indicates a header lines.  The default
            is 'H'.
        read : boolean, optional
            Boolean of weather this is reading or writing a file.  The
            default is True.
        headerlines : list, optional
            Additional list of strings of header data, only used in
            trajectory data Types. The default is [].
        '''
        self.sldir=sldir
        self.files = []
        self.header_char=header_char
        self.hattrs=[]
        self.dcols=[]
        self.data ={}
        self.headerLines=[]
        self.files.append(filename)
        self.datatype=datatype
        if 'trajectory' in filename or 'Trajectory' in filename:
            self.datatype='trajectory'

        if read:

            self.hattrs,self.data=self._readFile(sldir,filename,sep)
            self.dcols=self.data.keys()
        else:
            a=self.write(filename,headers,dcols,data,headerlines,sldir,sep)
            if a ==None:
                return None
            self.hattrs,self.data=self._readFile(sldir,filename,sep)
        self.dcols=self.data.keys()

    def get(self, attri):
        ''' 
        Method that dynamically determines the type of attribute that is
        passed into this method. Also it then returns that attribute's
        associated data.
        
        Parameters
        ----------
        attri : string 
            The attribute we are looking for.
        
        '''
        isCol=False
        isHead=False

        if attri in self.dcols:
            isCol=True
        elif attri in self.hattrs:
            isHead=True
        else:
            print "That attribute does not exist in this File"
            print 'Returning None'

        if isCol:
            return self.getColData(attri)
        elif isHead:
            return hattrs

    def getColData(self, attri):
        ''' 
        Method that returns column data

        Parameters
        ----------
        attri : string
            The attribute we are looking for.

        '''
        return self.data[attri]

    def _readFile(self, sldir, fileName, sep):
        ''' 
        Private method that reads in the header and column data.

        '''

        if sldir.endswith(os.sep):
            fileName = str(sldir)+str(fileName)
        else:
            fileName = str(sldir)+os.sep+str(fileName)


        fileLines=[] #list of lines in the file
        header=[]    #list of Header lines
        dataCols=[]  #Dictionary of data column names
        data=[]      #List of Data lists
        cols=[]      #List of column names

        f=open(fileName,'r')
        fileLines=f.readlines()
        i=0
        if self.datatype != 'trajectory':

            while i<len(fileLines):
                if fileLines[i].startswith(self.header_char):
                    tmp=fileLines[i].lstrip(self.header_char)
                    header.append(tmp.strip())
                else:
                    break
                i+=1

            cols=fileLines[i].split(sep)

            tmp=[]
            tmp1=[]
            for j in range(len(cols)):
                tmp1=cols[j].strip()
                if tmp1 !='':
                    tmp.append(tmp1)
            cols=tmp
            i+=1
        else:
            header={}
            while fileLines[i].startswith('#') or '=' in fileLines[i]:
                if fileLines[i].startswith('#') and cols==[]:
                    cols=fileLines[i].strip('#')
                    cols=cols.strip()
                    cols=cols.split()
                elif fileLines[i].startswith('#'):
                    tmp1=fileLines[i].strip('#')
                    tmp1=tmp1.strip()
                    self.headerLines.append(tmp1)
                elif not fileLines[i].startswith('#'):
                    tmp=fileLines[i].split('=')
                    tmp[0]=tmp[0].strip()
                    tmp[1]=tmp[1].strip()
                    if header=={}:
                        header={str(tmp[0]):str(tmp[1])}
                    else:
                        header[str(tmp[0])]=str(tmp[1])
                i+=1
        while i<len(fileLines):
            tmp=fileLines[i].split()
            for j in xrange(len(tmp)):
                tmp[j]=tmp[j].strip()
            data.append(tmp)
            i+=1
        tmp=[]
        tmp1=[]
        for j in range(len(data)):
            for k in range(len(data[j])):
                tmp1=data[j][k].strip()
                if tmp1 !='':
                    tmp.append(tmp1)
            data[j]=tmp
            tmp=[]
        tmp=[]

        for j in range(len(cols)):
            for k in range(len(data)):
                tmp.append(float(data[k][j]))
            tmp=array(tmp)

            if j == 0:
                dataCols={cols[j]:tmp}
            else:
                dataCols[cols[j]]=tmp
            tmp=[]

        return header,dataCols

#Global methods
def writeTraj(filename='trajectory.input', data=[], ageunit=0, tunit=0,
              rhounit=0, idNum=0):
    ''' 
    Method for writeing Trajectory type ascii files files.

    Parameters
    ----------
    filename : string
        The file where this data will be written.
    data : list
        A list of 1D data vectors with time, T and rho.
    ageunit : integer, optional
        If 1 ageunit = SEC, If 0 ageunit = YRS. If 2 agunit =
        logtimerev in yrs.  The default is 0.  logtimerev is log of
        time until end
    tunit : integer, optional
        If 1 TUNIT  = T9K, if 0 TUNIT = T8K.  The default is 0.
    rhounit : integer, optional
        If 1 RHOUNIT = LOG, if 0 RHOUNIT = CGS. The default is 0.
    idNum : optional
        An optional id argument

    '''
    if data==[]:
        print 'Please input correct data'
        print 'returning None'
        return None
    headers=[]
    if ageunit ==1:
        headers.append('AGEUNIT = SEC')
    elif ageunit==0:
        headers.append('AGEUNIT = YRS')
    elif ageunit==2:
        headers.append('AGEUNIT = logtimerev/yrs')

    if tunit ==1:
        headers.append('TUNIT   = T9K')
    elif tunit==0:
        headers.append('TUNIT   = T8K')

    if rhounit ==1:
        headers.append('RHOUNIT = LOG')
    elif rhounit==0:
        headers.append('RHOUNIT = CGS')

    headers.append('ID = '+str(idNum))

    write(filename,headers,['time','T','rho'],data,['YRS/SEC; T8K/T9K; CGS/LOG',"FORMAT: '(10x,A3)'"],trajectory=True)

def write(filename, headers, dcols, data, headerlines=[],
          header_char='H', sldir='.', sep='  ', trajectory=False,
          download=False):
    ''' 
    Method for writeing Ascii files.
    
    Note the attribute name at position i in dcols will be associated
    with the column data at index i in data.  Also the number of data
    columns(in data) must equal the number of data attributes (in dcols)
    Also all the lengths of that columns must all be the same.
    
    Parameters
    ----------
    filename : string
        The file where this data will be written.
    Headers : list
        A list of Header strings or if the file being written is of
        type trajectory, this is a List of strings that contain header
        attributes and their associated values which are seperated by
        a '='.
    dcols : list
        A list of data attributes.
    data : list
        A list of lists (or of numpy arrays).
    headerlines : list, optional
        Additional list of strings of header data, only used in
        trajectory data Types. The default is [].
    header_char : character, optional
        The character that indicates a header lines. The default is 'H'.
    sldir : string, optional
        Where this fill will be written. The default is '.'.
    sep : string, optional
        What seperates the data column attributes. The default is '  '.
    trajectory : boolean, optional
        Boolean of if we are writeing a trajectory type file. The
        default is False.
    download : boolean, optional
        If using iPython notebook, do you want a download link for
        the file you write?
        The default is False.
        
    '''
    if sldir.endswith(os.sep):
        filename = str(sldir)+str(filename)
    else:
        filename = str(sldir)+os.sep+str(filename)
    tmp=[] #temp variable
    lines=[]#list of the data lines
    lengthList=[]# list of the longest element (data or column name)
                 # in each column

    if os.path.exists(filename):
        print 'Warning this method will overwrite '+ filename
        print 'Would you like to continue? (y)es or (n)no?'
        s = raw_input('--> ')
        if s=='Y' or s=='y' or s=='Yes' or s=='yes':
            print 'Yes selected'
            print 'Continuing as normal'
        else:
            print 'No Selected'
            print 'Returning None'
            return None

    if len(data)!=len(dcols):
        print 'The number of data columns does not equal the number of Data attributes'
        print 'returning none'
        return None
    if trajectory:
        sep=' '
    for i in xrange(len(headers)):
        if not trajectory:
            tmp.append(header_char+' '+headers[i]+'\n')
        else:
            tmp.append(headers[i]+'\n')
    headers=tmp
    tmp=''

    for i in xrange(len(data)): #Line length stuff
        length=len(dcols[i])
        for j in xrange(len(data[i])):
            if len(str(data[i][j]))>length:
                length=len(str(data[i][j]))
        lengthList.append(length)

    tmp=''
    tmp1=''
    if trajectory:
        tmp='#'
    for i in xrange(len(dcols)):
        tmp1=dcols[i]
        if not trajectory:
            if len(dcols[i]) < lengthList[i]:
                j=lengthList[i]-len(dcols[i])
                for k in xrange(j):
                    tmp1+=' '
            tmp+=sep+tmp1
        else:
            tmp+=' '+dcols[i]
    tmp+='\n'
    dcols=tmp
    tmp=''


    for i in xrange(len(data[0])):
        for j in xrange(len(data)):
            tmp1=str(data[j][i])
            if len(str(data[j][i])) < lengthList[j]:
                l=lengthList[j]-len(str(data[j][i]))
                for k in xrange(l):
                    tmp1+=' '
            tmp+=sep+tmp1
        lines.append(tmp+'\n')
        tmp=''

    f=open(filename,'w')
    if not trajectory:
        for i in xrange(len(headers)):
            f.write(headers[i])
        f.write(dcols)
    else:
        f.write(dcols)
        for i in xrange(len(headerlines)):
            f.write('# '+headerlines[i]+'\n')
        for i in xrange(len(headers)):
            f.write(headers[i])
    for i in xrange(len(lines)):
        f.write(lines[i])

    f.close()
    if download:
        from IPython.display import FileLink, FileLinks
        return FileLink(filename)
    else:
        return None


def writeGCE_table(filename, headers, data, dcols=['Isotopes','Yields'],
                   header_char='H', sldir='.', sep='&'):
    ''' 
    Method for writeing data in GCE format in Ascii files.
    
    Note the attribute name at position i in dcols will be associated
    with the column data at index i in data.  Also the number of data
    columns(in data) must equal the number of data attributes (in dcols)
    Also all the lengths of that columns must all be the same.
    
    Parameters
    ----------
    filename : string
        The file where this data will be written.
    Headers : list
        A list of Header strings or if the file being written is of
        type trajectory, this is a List of strings that contain header
        attributes and their associated values which are seperated by
        a '='.
    data : list
        A list of lists (or of numpy arrays).
    dcols : list, optional
        A list of data attributes. The default is ['Isotopes','Yields'].
    header_char : character, optional
        The character that indicates a header lines. The default is 'H'.
    sldir : string, optional
        Where this fill will be written. The default is '.'.
    sep : string, optional
        What seperatesa the data column attributes. The default is '&'.

    '''

    import re

    if sldir.endswith(os.sep):
        filename = str(sldir)+str(filename)
    else:
        filename = str(sldir)+os.sep+str(filename)
    tmp=[] #temp variable
    lines=[]#list of the data lines
    lengthList=[]# list of the longest element (data or column name)
                 # in each column

    if os.path.exists(filename):
        print 'This method will add table to existing file '+ filename

    if len(data)!=len(dcols):
        print 'The number of data columns does not equal the number of Data attributes'
        print 'returning none'
        return None
    for i in xrange(len(headers)):
        tmp.append(header_char+' '+headers[i]+'\n')
    headers=tmp
    tmp=''

    for i in xrange(len(data)): #Line length stuff
        length=len(dcols[i])+1
        for j in xrange(len(data[i])):
            if len(str(data[i][j]))>length:
                length=len(str(data[i][j]))
        lengthList.append(length)

    tmp=''
    tmp1=''
    for i in xrange(len(dcols)):
        tmp1=dcols[i]
        if len(dcols[i]) < lengthList[i]:
            j=lengthList[i]-len(dcols[i])
            for k in xrange(j):
                tmp1+=' '
        tmp+=sep+tmp1
    tmp+='\n'
    dcols=tmp
    tmp=''


    for i in xrange(len(data[0])):
        for j in xrange(len(data)):
            if type(data[j][i]) == str:
                match = re.match(r"([a-z]+)([0-9]+)",data[j][i], re.I)
                items = match.groups()
                tmp1=items[0].capitalize()+'-'+items[1]
                if len(str(data[j][i])) < lengthList[j]:
                    l=lengthList[j]-len(tmp1)
                    for k in xrange(l):
                        tmp1+=' '
            else:
                tmp1='{:.3E}'.format(data[j][i])
                if len('{:.3E}'.format(data[j][i])) < lengthList[j]:
                    l=lengthList[j]-len('{:.4E}'.format(data[j][i]))
                    for k in xrange(l):
                        tmp1+=' '
            tmp+=sep+tmp1
        lines.append(tmp+'\n')
        tmp=''

    f=open(filename,'a')
    for i in xrange(len(headers)):
        f.write(headers[i])
    f.write(dcols)
    for i in xrange(len(lines)):
        f.write(lines[i])

    f.close()
    return None



#!/usr/bin/env python

# python distutils.core.setup reference at
# "https://docs.python.org/2/distutils/apiref.html"
from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES

# install data_files with the source
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

setup(name='NuGridpy',
    version='0.1.4644', 
    description='Python tools for NuGrid',
    long_description=open('README', 'r').read(), # get discription from README
    author='NuGrid Team',
    author_email='swjones@uvic.ca',
    url='http://www.nugridstars.org/',
    #download_url='http://www.nugridstars.org/', #must be uptodate if used
    py_modules=['ppn', 'ppm', 'nugridse', 'mesa', 'data_plot', 'utils',
                'ascii_table', 'h5T','grain'],
    # https://pypi.python.org/pypi?:action=list_classifiers : PyPI classifiers.
    classifiers=['Development Status :: 5 - Production/Stable',
                 'Environment :: Console', 'Framework :: IPython',
                 'Intended Audience :: Science/Research',
                 'License :: OSI Approved :: BSD License',
                 'Natural Language :: English',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python :: 2.7',
                 'Topic :: Scientific/Engineering :: Astronomy',
                 'Topic :: Scientific/Engineering :: Visualization'],
    license='BSD 3-clause',
    platforms=['Linux', 'Windows', 'OS X'],
    data_files=[('./NuGridpy', ['LICENSE', 'README', 'AUTHORS'])])
